"""
Test completing types using information from other shared libraries.
"""

import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LimitDebugInfoTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def _check_type(self, target, name):
        exe = target.FindModule(lldb.SBFileSpec("a.out"))
        type_ = exe.FindFirstType(name)
        self.trace("type_: %s"%type_)
        self.assertTrue(type_)
        base = type_.GetDirectBaseClassAtIndex(0).GetType()
        self.trace("base:%s"%base)
        self.assertTrue(base)
        self.assertEquals(base.GetNumberOfFields(), 0)

    def _check_debug_info_is_limited(self, target):
        # Without other shared libraries we should only see the member declared
        # in the derived class. This serves as a sanity check that we are truly
        # building with limited debug info.
        self._check_type(target, "InheritsFromOne")
        self._check_type(target, "InheritsFromTwo")

    @skipIf(bugnumber="pr46284", debug_info="gmodules")
    @skipIfWindows # Clang emits type info even with -flimit-debug-info
    # Requires DW_CC_pass_by_* attributes from Clang 7 to correctly call
    # by-value functions.
    @skipIf(compiler="clang", compiler_version=['<', '7.0'])
    def test_one_and_two_debug(self):
        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))

        self._check_debug_info_is_limited(target)

        lldbutil.run_to_name_breakpoint(self, "main",
                extra_images=["one", "two"])

        # But when other shared libraries are loaded, we should be able to see
        # all members.
        self.expect_expr("inherits_from_one.member", result_value="47")
        self.expect_expr("inherits_from_one.one", result_value="142")
        self.expect_expr("inherits_from_two.member", result_value="47")
        self.expect_expr("inherits_from_two.one", result_value="142")
        self.expect_expr("inherits_from_two.two", result_value="242")

        self.expect_expr("one_as_member.member", result_value="47")
        self.expect_expr("one_as_member.one.member", result_value="147")
        self.expect_expr("two_as_member.member", result_value="47")
        self.expect_expr("two_as_member.two.one.member", result_value="147")
        self.expect_expr("two_as_member.two.member", result_value="247")

        self.expect_expr("array_of_one[2].member", result_value="174")
        self.expect_expr("array_of_two[2].one[2].member", result_value="174")
        self.expect_expr("array_of_two[2].member", result_value="274")

        self.expect_expr("get_one().member", result_value="124")
        self.expect_expr("get_two().one().member", result_value="124")
        self.expect_expr("get_two().member", result_value="224")

        self.expect_expr("shadowed_one.member", result_value="47")
        self.expect_expr("shadowed_one.one", result_value="142")

    @skipIf(bugnumber="pr46284", debug_info="gmodules")
    @skipIfWindows # Clang emits type info even with -flimit-debug-info
    # Requires DW_CC_pass_by_* attributes from Clang 7 to correctly call
    # by-value functions.
    @skipIf(compiler="clang", compiler_version=['<', '7.0'])
    def test_two_debug(self):
        self.build(dictionary=dict(STRIP_ONE="1"))
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))

        self._check_debug_info_is_limited(target)

        lldbutil.run_to_name_breakpoint(self, "main",
                extra_images=["one", "two"])

        # This time, we should only see the members from the second library.
        self.expect_expr("inherits_from_one.member", result_value="47")
        self.expect("expr inherits_from_one.one", error=True,
            substrs=["no member named 'one' in 'InheritsFromOne'"])
        self.expect_expr("inherits_from_two.member", result_value="47")
        self.expect("expr inherits_from_two.one", error=True,
            substrs=["no member named 'one' in 'InheritsFromTwo'"])
        self.expect_expr("inherits_from_two.two", result_value="242")

        self.expect_expr("one_as_member.member", result_value="47")
        self.expect("expr one_as_member.one.member", error=True,
                substrs=["no member named 'member' in 'member::One'"])
        self.expect_expr("two_as_member.member", result_value="47")
        self.expect("expr two_as_member.two.one.member", error=True,
                substrs=["no member named 'member' in 'member::One'"])
        self.expect_expr("two_as_member.two.member", result_value="247")

        self.expect("expr array_of_one[2].member", error=True,
                substrs=["no member named 'member' in 'array::One'"])
        self.expect("expr array_of_two[2].one[2].member", error=True,
                substrs=["no member named 'member' in 'array::One'"])
        self.expect_expr("array_of_two[2].member", result_value="274")

        self.expect("expr get_one().member", error=True,
                substrs=["calling 'get_one' with incomplete return type 'result::One'"])
        self.expect("expr get_two().one().member", error=True,
                substrs=["calling 'one' with incomplete return type 'result::One'"])
        self.expect_expr("get_two().member", result_value="224")

    @skipIf(bugnumber="pr46284", debug_info="gmodules")
    @skipIfWindows # Clang emits type info even with -flimit-debug-info
    # Requires DW_CC_pass_by_* attributes from Clang 7 to correctly call
    # by-value functions.
    @skipIf(compiler="clang", compiler_version=['<', '7.0'])
    def test_one_debug(self):
        self.build(dictionary=dict(STRIP_TWO="1"))
        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))

        self._check_debug_info_is_limited(target)

        lldbutil.run_to_name_breakpoint(self, "main",
                extra_images=["one", "two"])

        # In this case we should only see the members from the second library.
        # Note that we cannot see inherits_from_two.one because without debug
        # info for "Two", we cannot determine that it in fact inherits from
        # "One".
        self.expect_expr("inherits_from_one.member", result_value="47")
        self.expect_expr("inherits_from_one.one", result_value="142")
        self.expect_expr("inherits_from_two.member", result_value="47")
        self.expect("expr inherits_from_two.one", error=True,
            substrs=["no member named 'one' in 'InheritsFromTwo'"])
        self.expect("expr inherits_from_two.two", error=True,
            substrs=["no member named 'two' in 'InheritsFromTwo'"])

        self.expect_expr("one_as_member.member", result_value="47")
        self.expect_expr("one_as_member.one.member", result_value="147")
        self.expect_expr("two_as_member.member", result_value="47")
        self.expect("expr two_as_member.two.one.member", error=True,
                substrs=["no member named 'one' in 'member::Two'"])
        self.expect("expr two_as_member.two.member", error=True,
                substrs=["no member named 'member' in 'member::Two'"])

        self.expect_expr("array_of_one[2].member", result_value="174")
        self.expect("expr array_of_two[2].one[2].member", error=True,
                substrs=["no member named 'one' in 'array::Two'"])
        self.expect("expr array_of_two[2].member", error=True,
                substrs=["no member named 'member' in 'array::Two'"])

        self.expect_expr("get_one().member", result_value="124")
        self.expect("expr get_two().one().member", error=True,
                substrs=["calling 'get_two' with incomplete return type 'result::Two'"])
        self.expect("expr get_two().member", error=True,
                substrs=["calling 'get_two' with incomplete return type 'result::Two'"])



"""
Test SBValue API linked_list_iter which treats the SBValue as a linked list and
supports iteration till the end of list is reached.
"""

from __future__ import print_function


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ValueAsLinkedListTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # We'll use the test method name as the exe_name.
        self.exe_name = self.testMethodName
        # Find the line number to break at.
        self.line = line_number('main.cpp', '// Break at this line')

    # Py3 asserts due to a bug in SWIG.  A fix for this was upstreamed into
    # SWIG 3.0.8.
    @skipIf(py_version=['>=', (3, 0)], swig_version=['<', (3, 0, 8)])
    def test(self):
        """Exercise SBValue API linked_list_iter."""
        d = {'EXE': self.exe_name}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        exe = self.getBuildArtifact(self.exe_name)

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Create the breakpoint inside function 'main'.
        breakpoint = target.BreakpointCreateByLocation('main.cpp', self.line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # Get Frame #0.
        self.assertEqual(process.GetState(), lldb.eStateStopped)
        thread = lldbutil.get_stopped_thread(
            process, lldb.eStopReasonBreakpoint)
        self.assertTrue(
            thread.IsValid(),
            "There should be a thread stopped due to breakpoint condition")
        frame0 = thread.GetFrameAtIndex(0)

        # Get variable 'task_head'.
        task_head = frame0.FindVariable('task_head')
        self.assertTrue(task_head, VALID_VARIABLE)
        self.DebugSBValue(task_head)

        # By design (see main.cpp), the visited id's are: [1, 2, 4, 5].
        visitedIDs = [1, 2, 4, 5]
        list = []

        cvf = lldbutil.ChildVisitingFormatter(indent_child=2)
        for t in task_head.linked_list_iter('next'):
            self.assertTrue(t, VALID_VARIABLE)
            # Make sure that 'next' corresponds to an SBValue with pointer
            # type.
            self.assertTrue(t.TypeIsPointerType())
            if self.TraceOn():
                print(cvf.format(t))
            list.append(int(t.GetChildMemberWithName("id").GetValue()))

        # Sanity checks that the we visited all the items (no more, no less).
        if self.TraceOn():
            print("visited IDs:", list)
        self.assertEqual(visitedIDs, list)

        # Let's exercise the linked_list_iter() API again, this time supplying
        # our end of list test function.
        def eol(val):
            """Test function to determine end of list."""
            # End of list is reached if either the value object is invalid
            # or it corresponds to a null pointer.
            if not val or int(val.GetValue(), 16) == 0:
                return True
            # Also check the "id" for correct semantics.  If id <= 0, the item
            # is corrupted, let's return True to signify end of list.
            if int(val.GetChildMemberWithName("id").GetValue(), 0) <= 0:
                return True

            # Otherwise, return False.
            return False

        list = []
        for t in task_head.linked_list_iter('next', eol):
            self.assertTrue(t, VALID_VARIABLE)
            # Make sure that 'next' corresponds to an SBValue with pointer
            # type.
            self.assertTrue(t.TypeIsPointerType())
            if self.TraceOn():
                print(cvf.format(t))
            list.append(int(t.GetChildMemberWithName("id").GetValue()))

        # Sanity checks that the we visited all the items (no more, no less).
        if self.TraceOn():
            print("visited IDs:", list)
        self.assertEqual(visitedIDs, list)

        # Get variable 'empty_task_head'.
        empty_task_head = frame0.FindVariable('empty_task_head')
        self.assertTrue(empty_task_head, VALID_VARIABLE)
        self.DebugSBValue(empty_task_head)

        list = []
        # There is no iterable item from empty_task_head.linked_list_iter().
        for t in empty_task_head.linked_list_iter('next', eol):
            if self.TraceOn():
                print(cvf.format(t))
            list.append(int(t.GetChildMemberWithName("id").GetValue()))

        self.assertEqual(len(list), 0)

        # Get variable 'task_evil'.
        task_evil = frame0.FindVariable('task_evil')
        self.assertTrue(task_evil, VALID_VARIABLE)
        self.DebugSBValue(task_evil)

        list = []
        # There 3 iterable items from task_evil.linked_list_iter(). :-)
        for t in task_evil.linked_list_iter('next'):
            if self.TraceOn():
                print(cvf.format(t))
            list.append(int(t.GetChildMemberWithName("id").GetValue()))

        self.assertEqual(len(list), 3)



"""Test stepping and setting breakpoints in indirect and re-exported symbols."""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestIndirectFunctions(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers that we will step to in main:
        self.main_source = "main.c"

    @skipUnlessDarwin
    @add_test_categories(['pyapi'])
    def test_with_python_api(self):
        """Test stepping and setting breakpoints in indirect and re-exported symbols."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        lib1 = self.getBuildArtifact('libindirect.dylib')
        lib2 = self.getBuildArtifact('libreexport.dylib')
        self.registerSharedLibrariesWithTarget(target, [lib1, lib2])

        self.main_source_spec = lldb.SBFileSpec(self.main_source)

        break1 = target.BreakpointCreateBySourceRegex(
            "Set breakpoint here to step in indirect.", self.main_source_spec)
        self.assertTrue(break1, VALID_BREAKPOINT)

        break2 = target.BreakpointCreateBySourceRegex(
            "Set breakpoint here to step in reexported.", self.main_source_spec)
        self.assertTrue(break2, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break1)
        if len(threads) != 1:
            self.fail("Failed to stop at breakpoint 1.")

        thread = threads[0]

        # Now do a step-into, and we should end up in the hidden target of this
        # indirect function.
        thread.StepInto()
        curr_function = thread.GetFrameAtIndex(0).GetFunctionName()
        self.assertEqual(curr_function, "call_through_indirect_hidden",
            "Stepped into indirect symbols.")

        # Now set a breakpoint using the indirect symbol name, and make sure we
        # get to that:
        break_indirect = target.BreakpointCreateByName("call_through_indirect")
        self.assertTrue(break_indirect, VALID_BREAKPOINT)

        # Now continue should take us to the second call through the indirect
        # symbol:

        threads = lldbutil.continue_to_breakpoint(process, break_indirect)
        self.assertEqual(
            len(threads), 1,
            "Stopped at breakpoint in indirect function.")
        curr_function = thread.GetFrameAtIndex(0).GetFunctionName()
        self.assertEqual(
            curr_function, "call_through_indirect_hidden",
            "Stepped into indirect symbols.")

        # Delete this breakpoint so it won't get in the way:
        target.BreakpointDelete(break_indirect.GetID())

        # Now continue to the site of the first re-exported function call in
        # main:
        threads = lldbutil.continue_to_breakpoint(process, break2)

        # This is stepping Into through a re-exported symbol to an indirect
        # symbol:
        thread.StepInto()
        curr_function = thread.GetFrameAtIndex(0).GetFunctionName()
        self.assertEqual(
            curr_function, "call_through_indirect_hidden",
            "Stepped into indirect symbols.")

        # And the last bit is to set a breakpoint on the re-exported symbol and
        # make sure we are again in out target function.
        break_reexported = target.BreakpointCreateByName(
            "reexport_to_indirect")
        self.assertTrue(break_reexported, VALID_BREAKPOINT)

        # Now continue should take us to the second call through the indirect
        # symbol:

        threads = lldbutil.continue_to_breakpoint(process, break_reexported)
        self.assertEqual(
            len(threads), 1,
            "Stopped at breakpoint in reexported function target.")
        curr_function = thread.GetFrameAtIndex(0).GetFunctionName()
        self.assertEqual(
            curr_function, "call_through_indirect_hidden",
            "Stepped into indirect symbols.")



"""
Test that we handle breakpoints on consecutive instructions correctly.
"""



import unittest2
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ConsecutiveBreakpointsTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def prepare_test(self):
        self.build()

        (self.target, self.process, self.thread, bkpt) = lldbutil.run_to_source_breakpoint(
                self, "Set breakpoint here", lldb.SBFileSpec("main.cpp"))

        # Set breakpoint to the next instruction
        frame = self.thread.GetFrameAtIndex(0)

        address = frame.GetPCAddress()
        instructions = self.target.ReadInstructions(address, 2)
        self.assertEquals(len(instructions), 2)
        self.bkpt_address = instructions[1].GetAddress()
        self.breakpoint2 = self.target.BreakpointCreateByAddress(
            self.bkpt_address.GetLoadAddress(self.target))
        self.assertTrue(
            self.breakpoint2 and self.breakpoint2.GetNumLocations() == 1,
            VALID_BREAKPOINT)

    def finish_test(self):
        # Run the process until termination
        self.process.Continue()
        self.assertEquals(self.process.GetState(), lldb.eStateExited)

    @no_debug_info_test
    def test_continue(self):
        """Test that continue stops at the second breakpoint."""
        self.prepare_test()

        self.process.Continue()
        self.assertEquals(self.process.GetState(), lldb.eStateStopped)
        # We should be stopped at the second breakpoint
        self.thread = lldbutil.get_one_thread_stopped_at_breakpoint(
            self.process, self.breakpoint2)
        self.assertIsNotNone(
            self.thread,
            "Expected one thread to be stopped at breakpoint 2")

        self.finish_test()

    @no_debug_info_test
    def test_single_step(self):
        """Test that single step stops at the second breakpoint."""
        self.prepare_test()

        step_over = False
        self.thread.StepInstruction(step_over)

        self.assertEquals(self.process.GetState(), lldb.eStateStopped)
        self.assertEquals(
            self.thread.GetFrameAtIndex(0).GetPCAddress().GetLoadAddress(
                self.target), self.bkpt_address.GetLoadAddress(
                self.target))
        self.thread = lldbutil.get_one_thread_stopped_at_breakpoint(
            self.process, self.breakpoint2)
        self.assertIsNotNone(
            self.thread,
            "Expected one thread to be stopped at breakpoint 2")

        self.finish_test()

    @no_debug_info_test
    def test_single_step_thread_specific(self):
        """Test that single step stops, even though the second breakpoint is not valid."""
        self.prepare_test()

        # Choose a thread other than the current one. A non-existing thread is
        # fine.
        thread_index = self.process.GetNumThreads() + 1
        self.assertFalse(self.process.GetThreadAtIndex(thread_index).IsValid())
        self.breakpoint2.SetThreadIndex(thread_index)

        step_over = False
        self.thread.StepInstruction(step_over)

        self.assertEquals(self.process.GetState(), lldb.eStateStopped)
        self.assertEquals(
            self.thread.GetFrameAtIndex(0).GetPCAddress().GetLoadAddress(
                self.target), self.bkpt_address.GetLoadAddress(
                self.target))
        self.assertEquals(
            self.thread.GetStopReason(),
            lldb.eStopReasonPlanComplete,
            "Stop reason should be 'plan complete'")

        self.finish_test()



"""
Test getting return-values correctly when stepping out
"""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ReturnValueTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def affected_by_pr33042(self):
        return ("clang" in self.getCompiler() and self.isAArch64() and
            self.getPlatform() == "linux")

    def affected_by_pr44132(self):
        return (self.getArchitecture() in ["aarch64", "arm"] and
                self.getPlatform() in ["freebsd", "linux"])

    # ABIMacOSX_arm(64) can't fetch simple values inside a structure
    def affected_by_radar_34562999(self):
        arch = self.getArchitecture().lower()
        return arch in ['arm64', 'arm64e', 'armv7', 'armv7k'] and self.platformIsDarwin()

    @expectedFailureAll(oslist=["freebsd"], archs=["i386"],
                        bugnumber="llvm.org/pr48376")
    @expectedFailureAll(oslist=["macosx"], archs=["i386"], bugnumber="<rdar://problem/28719652>")
    @expectedFailureAll(
        oslist=["linux"],
        compiler="clang",
        compiler_version=[
            "<=",
            "3.6"],
        archs=["i386"])
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24778")
    @add_test_categories(['pyapi'])
    def test_with_python(self):
        """Test getting return values from stepping out."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        (self.target, self.process, thread, inner_sint_bkpt) = lldbutil.run_to_name_breakpoint(self, "inner_sint", exe_name = exe)

        error = lldb.SBError()

        # inner_sint returns the variable value, so capture that here:
        in_int = thread.GetFrameAtIndex(0).FindVariable(
            "value").GetValueAsSigned(error)
        self.assertTrue(error.Success())

        thread.StepOut()

        self.assertEquals(self.process.GetState(), lldb.eStateStopped)
        self.assertEquals(thread.GetStopReason(), lldb.eStopReasonPlanComplete)

        frame = thread.GetFrameAtIndex(0)
        fun_name = frame.GetFunctionName()
        self.assertEquals(fun_name, "outer_sint(int)")

        return_value = thread.GetStopReturnValue()
        self.assertTrue(return_value.IsValid())

        ret_int = return_value.GetValueAsSigned(error)
        self.assertTrue(error.Success())
        self.assertEquals(in_int, ret_int)

        # Run again and we will stop in inner_sint the second time outer_sint is called.
        # Then test stepping out two frames at once:

        thread_list = lldbutil.continue_to_breakpoint(self.process, inner_sint_bkpt)
        self.assertEquals(len(thread_list), 1)
        thread = thread_list[0]

        # We are done with the inner_sint breakpoint:
        self.target.BreakpointDelete(inner_sint_bkpt.GetID())

        frame = thread.GetFrameAtIndex(1)
        fun_name = frame.GetFunctionName()
        self.assertEquals(fun_name, "outer_sint(int)")
        in_int = frame.FindVariable("value").GetValueAsSigned(error)
        self.assertTrue(error.Success())

        thread.StepOutOfFrame(frame)

        self.assertEquals(self.process.GetState(), lldb.eStateStopped)
        self.assertEquals(thread.GetStopReason(), lldb.eStopReasonPlanComplete)
        frame = thread.GetFrameAtIndex(0)
        fun_name = frame.GetFunctionName()
        self.assertEquals(fun_name, "main")

        ret_value = thread.GetStopReturnValue()
        self.assertTrue(return_value.IsValid())
        ret_int = ret_value.GetValueAsSigned(error)
        self.assertTrue(error.Success())
        self.assertEquals(2 * in_int, ret_int)

        # Now try some simple returns that have different types:
        inner_float_bkpt = self.target.BreakpointCreateByName(
            "inner_float(float)", exe)
        self.assertTrue(inner_float_bkpt, VALID_BREAKPOINT)
        self.process.Continue()
        thread_list = lldbutil.get_threads_stopped_at_breakpoint(
            self.process, inner_float_bkpt)
        self.assertEquals(len(thread_list), 1)
        thread = thread_list[0]

        self.target.BreakpointDelete(inner_float_bkpt.GetID())

        frame = thread.GetFrameAtIndex(0)
        in_value = frame.FindVariable("value")
        in_float = float(in_value.GetValue())
        thread.StepOut()

        self.assertEquals(self.process.GetState(), lldb.eStateStopped)
        self.assertEquals(thread.GetStopReason(), lldb.eStopReasonPlanComplete)

        frame = thread.GetFrameAtIndex(0)
        fun_name = frame.GetFunctionName()
        self.assertEquals(fun_name, "outer_float(float)")

        #return_value = thread.GetStopReturnValue()
        #self.assertTrue(return_value.IsValid())
        #return_float = float(return_value.GetValue())

        #self.assertEqual(in_float, return_float)

        if not self.affected_by_radar_34562999() and not self.affected_by_pr44132():
            self.return_and_test_struct_value("return_one_int")
            self.return_and_test_struct_value("return_two_int")
            self.return_and_test_struct_value("return_three_int")
            self.return_and_test_struct_value("return_four_int")
            if not self.affected_by_pr33042():
                self.return_and_test_struct_value("return_five_int")

            self.return_and_test_struct_value("return_two_double")
            self.return_and_test_struct_value("return_one_double_two_float")
            self.return_and_test_struct_value("return_one_int_one_float_one_int")

            self.return_and_test_struct_value("return_one_pointer")
            self.return_and_test_struct_value("return_two_pointer")
            self.return_and_test_struct_value("return_one_float_one_pointer")
            self.return_and_test_struct_value("return_one_int_one_pointer")
            self.return_and_test_struct_value("return_three_short_one_float")

            self.return_and_test_struct_value("return_one_int_one_double")
            self.return_and_test_struct_value("return_one_int_one_double_one_int")
            self.return_and_test_struct_value(
                "return_one_short_one_double_one_short")
            self.return_and_test_struct_value("return_one_float_one_int_one_float")
            self.return_and_test_struct_value("return_two_float")
            # I am leaving out the packed test until we have a way to tell CLANG
            # about alignment when reading DWARF for packed types.
            #self.return_and_test_struct_value ("return_one_int_one_double_packed")
            self.return_and_test_struct_value("return_one_int_one_long")

    @expectedFailureAll(oslist=["freebsd"], archs=["i386"],
                        bugnumber="llvm.org/pr48376")
    @expectedFailureAll(oslist=["macosx"], archs=["i386"], bugnumber="<rdar://problem/28719652>")
    @expectedFailureAll(
        oslist=["linux"],
        compiler="clang",
        compiler_version=[
            "<=",
            "3.6"],
        archs=["i386"])
    @expectedFailureAll(compiler=["gcc"], archs=["x86_64", "i386"])
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24778")
    @expectedFailureDarwin(archs=["arm64"]) # <rdar://problem/33976032> ABIMacOSX_arm64 doesn't get structs this big correctly
    def test_vector_values(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        error = lldb.SBError()

        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)

        main_bktp = self.target.BreakpointCreateByName("main", exe)
        self.assertTrue(main_bktp, VALID_BREAKPOINT)

        self.process = self.target.LaunchSimple(
            None, None, self.get_process_working_directory())
        self.assertEqual(len(lldbutil.get_threads_stopped_at_breakpoint(
            self.process, main_bktp)), 1)

        self.return_and_test_struct_value("return_vector_size_float32_8")
        self.return_and_test_struct_value("return_vector_size_float32_16")
        if not self.affected_by_pr44132():
            self.return_and_test_struct_value("return_vector_size_float32_32")
        self.return_and_test_struct_value("return_ext_vector_size_float32_2")
        self.return_and_test_struct_value("return_ext_vector_size_float32_4")
        if not self.affected_by_pr44132():
            self.return_and_test_struct_value("return_ext_vector_size_float32_8")

    # limit the nested struct and class tests to only x86_64
    @skipIf(archs=no_match(['x86_64']))
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24778")
    def test_for_cpp_support(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        (self.target, self.process, thread, inner_sint_bkpt) = lldbutil.run_to_name_breakpoint(self, "inner_sint", exe_name = exe)

        error = lldb.SBError()

        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)

        main_bktp = self.target.BreakpointCreateByName("main", exe)
        self.assertTrue(main_bktp, VALID_BREAKPOINT)

        self.process = self.target.LaunchSimple(
            None, None, self.get_process_working_directory())
        self.assertEqual(len(lldbutil.get_threads_stopped_at_breakpoint(
            self.process, main_bktp)), 1)
        # nested struct tests
        self.return_and_test_struct_value("return_nested_one_float_three_base")
        self.return_and_test_struct_value("return_double_nested_one_float_one_nested")
        self.return_and_test_struct_value("return_nested_float_struct")
        # class test
        self.return_and_test_struct_value("return_base_class_one_char")
        self.return_and_test_struct_value("return_nested_class_float_and_base")
        self.return_and_test_struct_value("return_double_nested_class_float_and_nested")
        self.return_and_test_struct_value("return_base_class")
        self.return_and_test_struct_value("return_derived_class")

    @skipIf(compiler="clang", compiler_version=['<', '7.0'])
    def return_and_test_struct_value(self, func_name):
        """Pass in the name of the function to return from - takes in value, returns value."""

        # Set the breakpoint, run to it, finish out.
        bkpt = self.target.BreakpointCreateByName(func_name)
        self.assertTrue(bkpt.GetNumResolvedLocations() > 0)

        self.process.Continue()

        thread_list = lldbutil.get_threads_stopped_at_breakpoint(
            self.process, bkpt)

        self.assertEquals(len(thread_list), 1)
        thread = thread_list[0]

        self.target.BreakpointDelete(bkpt.GetID())

        in_value = thread.GetFrameAtIndex(0).FindVariable("value")

        self.assertTrue(in_value.IsValid())
        num_in_children = in_value.GetNumChildren()

        # This is a little hokey, but if we don't get all the children now, then
        # once we've stepped we won't be able to get them?

        for idx in range(0, num_in_children):
            in_child = in_value.GetChildAtIndex(idx)
            in_child_str = in_child.GetValue()

        thread.StepOut()

        self.assertEquals(self.process.GetState(), lldb.eStateStopped)
        self.assertEquals(thread.GetStopReason(), lldb.eStopReasonPlanComplete)

        # Assuming all these functions step out to main.  Could figure out the caller dynamically
        # if that would add something to the test.
        frame = thread.GetFrameAtIndex(0)
        fun_name = frame.GetFunctionName()
        self.assertEquals(fun_name, "main")

        frame = thread.GetFrameAtIndex(0)
        ret_value = thread.GetStopReturnValue()

        self.assertTrue(ret_value.IsValid())

        num_ret_children = ret_value.GetNumChildren()
        self.assertEquals(num_in_children, num_ret_children)
        for idx in range(0, num_ret_children):
            in_child = in_value.GetChildAtIndex(idx)
            ret_child = ret_value.GetChildAtIndex(idx)
            in_child_str = in_child.GetValue()
            ret_child_str = ret_child.GetValue()

            self.assertEqual(in_child_str, ret_child_str)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class TestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @skipUnlessDarwin
    # LLDB ends up calling the user-defined function (but at least doesn't
    # crash).
    @expectedFailureDarwin
    def test(self):
        """
        Tests LLDB's behaviour if the user defines their own conflicting
        objc_copyRealizedClassList_nolock function.
        """

        self.build()
        lldbutil.run_to_source_breakpoint(self, "// break here", lldb.SBFileSpec("main.m"))

        # Get the (dynamic) type of our 'id' variable so that our Objective-C
        # runtime information is updated.
        str_val = self.expect_expr("custom_class")
        dyn_val = str_val.GetDynamicValue(lldb.eDynamicCanRunTarget)

        # We should have retrieved the proper class list even in presence of
        # the user-defined function.
        self.assertEqual(dyn_val.GetTypeName(), "CustomClass *")



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.passmanager import *

from mlir.dialects import sparse_tensor as st


def run(f):
  print('\nTEST:', f.__name__)
  f()
  return f


# CHECK-LABEL: TEST: testSparseTensorPass
@run
def testSparseTensorPass():
  with Context() as context:
    PassManager.parse('sparsification')
    PassManager.parse('sparse-tensor-conversion')
  # CHECK: SUCCESS
  print('SUCCESS')



"""Test lldb's startup delays creating a target, setting a breakpoint, and run to breakpoint stop."""

from __future__ import print_function


import sys
import lldb
from lldbsuite.test import configuration
from lldbsuite.test import lldbtest_config
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbbench import *


class StartupDelaysBench(BenchBase):

    mydir = TestBase.compute_mydir(__file__)

    def setUp(self):
        BenchBase.setUp(self)
        # Create self.stopwatch2 for measuring "set first breakpoint".
        # The default self.stopwatch is for "create fresh target".
        self.stopwatch2 = Stopwatch()
        # Create self.stopwatch3 for measuring "run to breakpoint".
        self.stopwatch3 = Stopwatch()
        self.exe = lldbtest_config.lldbExec
        self.break_spec = '-n main'
        self.count = 30

    @benchmarks_test
    @no_debug_info_test
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr22274: need a pexpect replacement for windows")
    def test_startup_delay(self):
        """Test start up delays creating a target, setting a breakpoint, and run to breakpoint stop."""
        print()
        self.run_startup_delays_bench(self.exe, self.break_spec, self.count)
        print(
            "lldb startup delay (create fresh target) benchmark:",
            self.stopwatch)
        print(
            "lldb startup delay (set first breakpoint) benchmark:",
            self.stopwatch2)
        print(
            "lldb startup delay (run to breakpoint) benchmark:",
            self.stopwatch3)

    def run_startup_delays_bench(self, exe, break_spec, count):
        import pexpect
        # Set self.child_prompt, which is "(lldb) ".
        self.child_prompt = '(lldb) '
        prompt = self.child_prompt

        # Reset the stopwatchs now.
        self.stopwatch.reset()
        self.stopwatch2.reset()
        for i in range(count):
            # So that the child gets torn down after the test.
            self.child = pexpect.spawn(
                '%s %s' %
                (lldbtest_config.lldbExec, self.lldbOption))
            child = self.child

            # Turn on logging for what the child sends back.
            if self.TraceOn():
                child.logfile_read = sys.stdout

            with self.stopwatch:
                # Create a fresh target.
                child.sendline('file %s' % exe)  # Aka 'target create'.
                child.expect_exact(prompt)

            with self.stopwatch2:
                # Read debug info and set the first breakpoint.
                child.sendline('breakpoint set %s' % break_spec)
                child.expect_exact(prompt)

            with self.stopwatch3:
                # Run to the breakpoint just set.
                child.sendline('run')
                child.expect_exact(prompt)

            child.sendline('quit')
            try:
                self.child.expect(pexpect.EOF)
            except:
                pass

        # The test is about to end and if we come to here, the child process has
        # been terminated.  Mark it so.
        self.child = None



"""
Test the IR interpreter
"""


import unittest2

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class IRInterpreterTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.c.
        self.line = line_number('main.c',
                                '// Set breakpoint here')

        # Disable confirmation prompt to avoid infinite wait
        self.runCmd("settings set auto-confirm true")
        self.addTearDownHook(
            lambda: self.runCmd("settings clear auto-confirm"))

    def build_and_run(self):
        """Test the IR interpreter"""
        self.build()

        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=1, loc_exact=False)

        self.runCmd("run", RUN_SUCCEEDED)

    @add_test_categories(['pyapi'])
    # getpid() is POSIX, among other problems, see bug
    @expectedFailureAll(
        oslist=['windows'],
        bugnumber="http://llvm.org/pr21765")
    def test_ir_interpreter(self):
        self.build_and_run()

        options = lldb.SBExpressionOptions()
        options.SetLanguage(lldb.eLanguageTypeC_plus_plus)

        set_up_expressions = ["int $i = 9", "int $j = 3", "int $k = 5",
            "unsigned long long $ull = -1", "unsigned $u = -1"]

        expressions = ["$i + $j",
                       "$i - $j",
                       "$i * $j",
                       "$i / $j",
                       "$i % $k",
                       "$i << $j",
                       "$i & $j",
                       "$i | $j",
                       "$i ^ $j",
                       "($ull & -1) == $u"]

        for expression in set_up_expressions:
            self.frame().EvaluateExpression(expression, options)

        for expression in expressions:
            interp_expression = expression
            jit_expression = "(int)getpid(); " + expression

            interp_result = self.frame().EvaluateExpression(
                interp_expression, options).GetValueAsSigned()
            jit_result = self.frame().EvaluateExpression(
                jit_expression, options).GetValueAsSigned()

            self.assertEqual(
                interp_result,
                jit_result,
                "While evaluating " +
                expression)

    def test_type_conversions(self):
        target = self.dbg.GetDummyTarget()
        short_val = target.EvaluateExpression("(short)-1")
        self.assertEqual(short_val.GetValueAsSigned(), -1)
        long_val = target.EvaluateExpression("(long) "+ short_val.GetName())
        self.assertEqual(long_val.GetValueAsSigned(), -1)



"""
Test the ptr_refs tool on Darwin with Objective-C
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestPtrRefsObjC(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @skipIfAsan # The output looks different under ASAN.
    def test_ptr_refs(self):
        """Test the ptr_refs tool on Darwin with Objective-C"""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        main_file_spec = lldb.SBFileSpec('main.m')
        breakpoint = target.BreakpointCreateBySourceRegex(
            'break', main_file_spec)
        self.assertTrue(breakpoint and
                        breakpoint.GetNumLocations() == 1,
                        VALID_BREAKPOINT)

        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # Frame #0 should be on self.line1 and the break condition should hold.
        thread = lldbutil.get_stopped_thread(
            process, lldb.eStopReasonBreakpoint)
        self.assertTrue(
            thread.IsValid(),
            "There should be a thread stopped due to breakpoint condition")

        frame = thread.GetFrameAtIndex(0)

        self.runCmd("command script import lldb.macosx.heap")
        self.expect("ptr_refs self", substrs=["malloc", "stack"])




"""
Test basics of Minidump debugging.
"""

from six import iteritems

import shutil

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class MiniDumpNewTestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    NO_DEBUG_INFO_TESTCASE = True

    _linux_x86_64_pid = 29917
    _linux_x86_64_not_crashed_pid = 29939
    _linux_x86_64_not_crashed_pid_offset = 0xD967

    def process_from_yaml(self, yaml_file):
        minidump_path = self.getBuildArtifact(os.path.basename(yaml_file) + ".dmp")
        self.yaml2obj(yaml_file, minidump_path)
        self.target = self.dbg.CreateTarget(None)
        self.process = self.target.LoadCore(minidump_path)
        return self.process

    def check_state(self):
        with open(os.devnull) as devnul:
            # sanitize test output
            self.dbg.SetOutputFileHandle(devnul, False)
            self.dbg.SetErrorFileHandle(devnul, False)

            self.assertTrue(self.process.is_stopped)

            # Process.Continue
            error = self.process.Continue()
            self.assertFalse(error.Success())
            self.assertTrue(self.process.is_stopped)

            # Thread.StepOut
            thread = self.process.GetSelectedThread()
            thread.StepOut()
            self.assertTrue(self.process.is_stopped)

            # command line
            self.dbg.HandleCommand('s')
            self.assertTrue(self.process.is_stopped)
            self.dbg.HandleCommand('c')
            self.assertTrue(self.process.is_stopped)

            # restore file handles
            self.dbg.SetOutputFileHandle(None, False)
            self.dbg.SetErrorFileHandle(None, False)

    def test_loadcore_error_status(self):
        """Test the SBTarget.LoadCore(core, error) overload."""
        minidump_path = self.getBuildArtifact("linux-x86_64.dmp")
        self.yaml2obj("linux-x86_64.yaml", minidump_path)
        self.target = self.dbg.CreateTarget(None)
        error = lldb.SBError()
        self.process = self.target.LoadCore(minidump_path, error)
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertTrue(error.Success())

    def test_loadcore_error_status_failure(self):
        """Test the SBTarget.LoadCore(core, error) overload."""
        self.target = self.dbg.CreateTarget(None)
        error = lldb.SBError()
        self.process = self.target.LoadCore("non-existent.dmp", error)
        self.assertFalse(self.process, PROCESS_IS_VALID)
        self.assertTrue(error.Fail())

    def test_process_info_in_minidump(self):
        """Test that lldb can read the process information from the Minidump."""
        self.process_from_yaml("linux-x86_64.yaml")
        self.assertTrue(self.process, PROCESS_IS_VALID)
        self.assertEqual(self.process.GetNumThreads(), 1)
        self.assertEqual(self.process.GetProcessID(), self._linux_x86_64_pid)
        self.check_state()

    def test_memory_region_name(self):
        self.process_from_yaml("regions-linux-map.yaml")
        result = lldb.SBCommandReturnObject()
        addr_region_name_pairs = [
            ("0x400d9000", "/system/bin/app_process"),
            ("0x400db000", "/system/bin/app_process"),
            ("0x400dd000", "/system/bin/linker"),
            ("0x400ed000", "/system/bin/linker"),
            ("0x400ee000", "/system/bin/linker"),
            ("0x400fb000", "/system/lib/liblog.so"),
            ("0x400fc000", "/system/lib/liblog.so"),
            ("0x400fd000", "/system/lib/liblog.so"),
            ("0x400ff000", "/system/lib/liblog.so"),
            ("0x40100000", "/system/lib/liblog.so"),
            ("0x40101000", "/system/lib/libc.so"),
            ("0x40122000", "/system/lib/libc.so"),
            ("0x40123000", "/system/lib/libc.so"),
            ("0x40167000", "/system/lib/libc.so"),
            ("0x40169000", "/system/lib/libc.so"),
        ]
        ci = self.dbg.GetCommandInterpreter()
        for (addr, region_name) in addr_region_name_pairs:
            command = 'memory region ' + addr
            ci.HandleCommand(command, result, False)
            message = 'Ensure memory "%s" shows up in output for "%s"' % (
                region_name, command)
            self.assertIn(region_name, result.GetOutput(), message)

    def test_thread_info_in_minidump(self):
        """Test that lldb can read the thread information from the Minidump."""
        self.process_from_yaml("linux-x86_64.yaml")
        self.check_state()
        # This process crashed due to a segmentation fault in its
        # one and only thread.
        self.assertEqual(self.process.GetNumThreads(), 1)
        thread = self.process.GetThreadAtIndex(0)
        self.assertEqual(thread.GetStopReason(), lldb.eStopReasonSignal)
        stop_description = thread.GetStopDescription(256)
        self.assertIn("SIGSEGV", stop_description)

    @skipIfLLVMTargetMissing("X86")
    def test_stack_info_in_minidump(self):
        """Test that we can see a trivial stack in a breakpad-generated Minidump."""
        # target create linux-x86_64 -c linux-x86_64.dmp
        self.dbg.CreateTarget("linux-x86_64")
        self.target = self.dbg.GetSelectedTarget()
        self.process = self.target.LoadCore("linux-x86_64.dmp")
        self.check_state()
        self.assertEqual(self.process.GetNumThreads(), 1)
        self.assertEqual(self.process.GetProcessID(), self._linux_x86_64_pid)
        thread = self.process.GetThreadAtIndex(0)
        # frame #0: linux-x86_64`crash()
        # frame #1: linux-x86_64`_start
        self.assertEqual(thread.GetNumFrames(), 2)
        frame = thread.GetFrameAtIndex(0)
        self.assertTrue(frame.IsValid())
        self.assertTrue(frame.GetModule().IsValid())
        pc = frame.GetPC()
        eip = frame.FindRegister("pc")
        self.assertTrue(eip.IsValid())
        self.assertEqual(pc, eip.GetValueAsUnsigned())

    def test_snapshot_minidump_dump_requested(self):
        """Test that if we load a snapshot minidump file (meaning the process
        did not crash) with exception code "DUMP_REQUESTED" there is no stop reason."""
        # target create -c linux-x86_64_not_crashed.dmp
        self.dbg.CreateTarget(None)
        self.target = self.dbg.GetSelectedTarget()
        self.process = self.target.LoadCore("linux-x86_64_not_crashed.dmp")
        self.check_state()
        self.assertEqual(self.process.GetNumThreads(), 1)
        thread = self.process.GetThreadAtIndex(0)
        self.assertEqual(thread.GetStopReason(), lldb.eStopReasonNone)
        stop_description = thread.GetStopDescription(256)
        self.assertEqual(stop_description, "")

    def test_snapshot_minidump_null_exn_code(self):
        """Test that if we load a snapshot minidump file (meaning the process
        did not crash) with exception code zero there is no stop reason."""
        self.process_from_yaml("linux-x86_64_null_signal.yaml")
        self.check_state()
        self.assertEqual(self.process.GetNumThreads(), 1)
        thread = self.process.GetThreadAtIndex(0)
        self.assertEqual(thread.GetStopReason(), lldb.eStopReasonNone)
        stop_description = thread.GetStopDescription(256)
        self.assertEqual(stop_description, "")

    def check_register_unsigned(self, set, name, expected):
        reg_value = set.GetChildMemberWithName(name)
        self.assertTrue(reg_value.IsValid(),
                        'Verify we have a register named "%s"' % (name))
        self.assertEqual(reg_value.GetValueAsUnsigned(), expected,
                         'Verify "%s" == %i' % (name, expected))

    def check_register_string_value(self, set, name, expected, format):
        reg_value = set.GetChildMemberWithName(name)
        self.assertTrue(reg_value.IsValid(),
                        'Verify we have a register named "%s"' % (name))
        if format is not None:
            reg_value.SetFormat(format)
        self.assertEqual(reg_value.GetValue(), expected,
                         'Verify "%s" has string value "%s"' % (name,
                                                                expected))

    def test_arm64_registers(self):
        """Test ARM64 registers from a breakpad created minidump."""
        self.process_from_yaml("arm64-macos.yaml")
        self.check_state()
        self.assertEqual(self.process.GetNumThreads(), 1)
        thread = self.process.GetThreadAtIndex(0)
        self.assertEqual(thread.GetStopReason(), lldb.eStopReasonNone)
        stop_description = thread.GetStopDescription(256)
        self.assertEqual(stop_description, "")
        registers = thread.GetFrameAtIndex(0).GetRegisters()
        # Verify the GPR registers are all correct
        # Verify x0 - x31 register values
        gpr = registers.GetValueAtIndex(0)
        for i in range(32):
            v = i+1 | i+2 << 32 | i+3 << 48
            w = i+1
            self.check_register_unsigned(gpr, 'x%i' % (i), v)
            self.check_register_unsigned(gpr, 'w%i' % (i), w)
        # Verify arg1 - arg8 register values
        for i in range(1, 9):
            v = i | i+1 << 32 | i+2 << 48
            self.check_register_unsigned(gpr, 'arg%i' % (i), v)
        i = 29
        v = i+1 | i+2 << 32 | i+3 << 48
        self.check_register_unsigned(gpr, 'fp', v)
        i = 30
        v = i+1 | i+2 << 32 | i+3 << 48
        self.check_register_unsigned(gpr, 'lr', v)
        i = 31
        v = i+1 | i+2 << 32 | i+3 << 48
        self.check_register_unsigned(gpr, 'sp', v)
        self.check_register_unsigned(gpr, 'pc', 0x1000)
        self.check_register_unsigned(gpr, 'cpsr', 0x11223344)
        self.check_register_unsigned(gpr, 'psr', 0x11223344)

        # Verify the FPR registers are all correct
        fpr = registers.GetValueAtIndex(1)
        for i in range(32):
            v = "0x"
            d = "0x"
            s = "0x"
            h = "0x"
            for j in range(i+15, i-1, -1):
                v += "%2.2x" % (j)
            for j in range(i+7, i-1, -1):
                d += "%2.2x" % (j)
            for j in range(i+3, i-1, -1):
                s += "%2.2x" % (j)
            for j in range(i+1, i-1, -1):
                h += "%2.2x" % (j)
            self.check_register_string_value(fpr, "v%i" % (i), v,
                                             lldb.eFormatHex)
            self.check_register_string_value(fpr, "d%i" % (i), d,
                                             lldb.eFormatHex)
            self.check_register_string_value(fpr, "s%i" % (i), s,
                                             lldb.eFormatHex)
            self.check_register_string_value(fpr, "h%i" % (i), h,
                                             lldb.eFormatHex)
        self.check_register_unsigned(gpr, 'fpsr', 0x55667788)
        self.check_register_unsigned(gpr, 'fpcr', 0x99aabbcc)

    def verify_arm_registers(self, apple=False):
        """
            Verify values of all ARM registers from a breakpad created
            minidump.
        """
        if apple:
            self.process_from_yaml("arm-macos.yaml")
        else:
            self.process_from_yaml("arm-linux.yaml")
        self.check_state()
        self.assertEqual(self.process.GetNumThreads(), 1)
        thread = self.process.GetThreadAtIndex(0)
        self.assertEqual(thread.GetStopReason(), lldb.eStopReasonNone)
        stop_description = thread.GetStopDescription(256)
        self.assertEqual(stop_description, "")
        registers = thread.GetFrameAtIndex(0).GetRegisters()
        # Verify the GPR registers are all correct
        # Verify x0 - x31 register values
        gpr = registers.GetValueAtIndex(0)
        for i in range(1, 16):
            self.check_register_unsigned(gpr, 'r%i' % (i), i+1)
        # Verify arg1 - arg4 register values
        for i in range(1, 5):
            self.check_register_unsigned(gpr, 'arg%i' % (i), i)
        if apple:
            self.check_register_unsigned(gpr, 'fp', 0x08)
        else:
            self.check_register_unsigned(gpr, 'fp', 0x0c)
        self.check_register_unsigned(gpr, 'lr', 0x0f)
        self.check_register_unsigned(gpr, 'sp', 0x0e)
        self.check_register_unsigned(gpr, 'pc', 0x10)
        self.check_register_unsigned(gpr, 'cpsr', 0x11223344)

        # Verify the FPR registers are all correct
        fpr = registers.GetValueAtIndex(1)
        # Check d0 - d31
        self.check_register_unsigned(gpr, 'fpscr', 0x55667788aabbccdd)
        for i in range(32):
            value = (i+1) | (i+1) << 8 | (i+1) << 32 | (i+1) << 48
            self.check_register_unsigned(fpr, "d%i" % (i), value)
        # Check s0 - s31
        for i in range(32):
            i_val = (i >> 1) + 1
            if i & 1:
                value = "%#8.8x" % (i_val | i_val << 16)
            else:
                value = "%#8.8x" % (i_val | i_val << 8)
            self.check_register_string_value(fpr, "s%i" % (i), value,
                                             lldb.eFormatHex)
        # Check q0 - q15
        for i in range(15):
            a = i * 2 + 1
            b = a + 1
            value = ("0x00%2.2x00%2.2x0000%2.2x%2.2x"
                     "00%2.2x00%2.2x0000%2.2x%2.2x") % (b, b, b, b, a, a, a, a)
            self.check_register_string_value(fpr, "q%i" % (i), value,
                                             lldb.eFormatHex)

    def test_linux_arm_registers(self):
        """Test Linux ARM registers from a breakpad created minidump.

           The frame pointer is R11 for linux.
        """
        self.verify_arm_registers(apple=False)

    def test_apple_arm_registers(self):
        """Test Apple ARM registers from a breakpad created minidump.

           The frame pointer is R7 for linux.
        """
        self.verify_arm_registers(apple=True)

    def do_test_deeper_stack(self, binary, core, pid):
        target = self.dbg.CreateTarget(binary)
        process = target.LoadCore(core)
        thread = process.GetThreadAtIndex(0)

        self.assertEqual(process.GetProcessID(), pid)

        expected_stack = {1: 'bar', 2: 'foo', 3: '_start'}
        self.assertGreaterEqual(thread.GetNumFrames(), len(expected_stack))
        for index, name in iteritems(expected_stack):
            frame = thread.GetFrameAtIndex(index)
            self.assertTrue(frame.IsValid())
            function_name = frame.GetFunctionName()
            self.assertIn(name, function_name)

    @skipIfLLVMTargetMissing("X86")
    def test_deeper_stack_in_minidump(self):
        """Test that we can examine a more interesting stack in a Minidump."""
        # Launch with the Minidump, and inspect the stack.
        # target create linux-x86_64_not_crashed -c linux-x86_64_not_crashed.dmp
        self.do_test_deeper_stack("linux-x86_64_not_crashed",
                                  "linux-x86_64_not_crashed.dmp",
                                  self._linux_x86_64_not_crashed_pid)

    def do_change_pid_in_minidump(self, core, newcore, offset, oldpid, newpid):
        """ This assumes that the minidump is breakpad generated on Linux -
        meaning that the PID in the file will be an ascii string part of
        /proc/PID/status which is written in the file
        """
        shutil.copyfile(core, newcore)
        with open(newcore, "rb+") as f:
            f.seek(offset)
            currentpid = f.read(5).decode('utf-8')
            self.assertEqual(currentpid, oldpid)

            f.seek(offset)
            if len(newpid) < len(oldpid):
                newpid += " " * (len(oldpid) - len(newpid))
            newpid += "\n"
            f.write(newpid.encode('utf-8'))

    @skipIfLLVMTargetMissing("X86")
    def test_deeper_stack_in_minidump_with_same_pid_running(self):
        """Test that we read the information from the core correctly even if we
        have a running process with the same PID"""
        new_core = self.getBuildArtifact("linux-x86_64_not_crashed-pid.dmp")
        self.do_change_pid_in_minidump("linux-x86_64_not_crashed.dmp",
                                       new_core,
                                       self._linux_x86_64_not_crashed_pid_offset,
                                       str(self._linux_x86_64_not_crashed_pid),
                                       str(os.getpid()))
        self.do_test_deeper_stack("linux-x86_64_not_crashed", new_core, os.getpid())

    @skipIfLLVMTargetMissing("X86")
    def test_two_cores_same_pid(self):
        """Test that we handle the situation if we have two core files with the same PID """
        new_core = self.getBuildArtifact("linux-x86_64_not_crashed-pid.dmp")
        self.do_change_pid_in_minidump("linux-x86_64_not_crashed.dmp",
                                       new_core,
                                       self._linux_x86_64_not_crashed_pid_offset,
                                       str(self._linux_x86_64_not_crashed_pid),
                                       str(self._linux_x86_64_pid))
        self.do_test_deeper_stack("linux-x86_64_not_crashed",
                                  new_core, self._linux_x86_64_pid)
        self.test_stack_info_in_minidump()

    @skipIfLLVMTargetMissing("X86")
    def test_local_variables_in_minidump(self):
        """Test that we can examine local variables in a Minidump."""
        # Launch with the Minidump, and inspect a local variable.
        # target create linux-x86_64_not_crashed -c linux-x86_64_not_crashed.dmp
        self.target = self.dbg.CreateTarget("linux-x86_64_not_crashed")
        self.process = self.target.LoadCore("linux-x86_64_not_crashed.dmp")
        self.check_state()
        thread = self.process.GetThreadAtIndex(0)
        frame = thread.GetFrameAtIndex(1)
        value = frame.EvaluateExpression('x')
        self.assertEqual(value.GetValueAsSigned(), 3)

    def test_memory_regions_in_minidump(self):
        """Test memory regions from a Minidump"""
        self.process_from_yaml("regions-linux-map.yaml")
        self.check_state()

        regions_count = 19
        region_info_list = self.process.GetMemoryRegions()
        self.assertEqual(region_info_list.GetSize(), regions_count)

        def check_region(index, start, end, read, write, execute, mapped, name):
            region_info = lldb.SBMemoryRegionInfo()
            self.assertTrue(
                self.process.GetMemoryRegionInfo(start, region_info).Success())
            self.assertEqual(start, region_info.GetRegionBase())
            self.assertEqual(end, region_info.GetRegionEnd())
            self.assertEqual(read, region_info.IsReadable())
            self.assertEqual(write, region_info.IsWritable())
            self.assertEqual(execute, region_info.IsExecutable())
            self.assertEqual(mapped, region_info.IsMapped())
            self.assertEqual(name, region_info.GetName())

            # Ensure we have the same regions as SBMemoryRegionInfoList contains.
            if index >= 0 and index < regions_count:
                region_info_from_list = lldb.SBMemoryRegionInfo()
                self.assertTrue(region_info_list.GetMemoryRegionAtIndex(
                    index, region_info_from_list))
                self.assertEqual(region_info_from_list, region_info)

        a = "/system/bin/app_process"
        b = "/system/bin/linker"
        c = "/system/lib/liblog.so"
        d = "/system/lib/libc.so"
        n = None
        max_int = 0xffffffffffffffff

        # Test address before the first entry comes back with nothing mapped up
        # to first valid region info
        check_region(-1, 0x00000000, 0x400d9000, False, False, False, False, n)
        check_region( 0, 0x400d9000, 0x400db000, True,  False, True,  True,  a)
        check_region( 1, 0x400db000, 0x400dc000, True,  False, False, True,  a)
        check_region( 2, 0x400dc000, 0x400dd000, True,  True,  False, True,  n)
        check_region( 3, 0x400dd000, 0x400ec000, True,  False, True,  True,  b)
        check_region( 4, 0x400ec000, 0x400ed000, True,  False, False, True,  n)
        check_region( 5, 0x400ed000, 0x400ee000, True,  False, False, True,  b)
        check_region( 6, 0x400ee000, 0x400ef000, True,  True,  False, True,  b)
        check_region( 7, 0x400ef000, 0x400fb000, True,  True,  False, True,  n)
        check_region( 8, 0x400fb000, 0x400fc000, True,  False, True,  True,  c)
        check_region( 9, 0x400fc000, 0x400fd000, True,  True,  True,  True,  c)
        check_region(10, 0x400fd000, 0x400ff000, True,  False, True,  True,  c)
        check_region(11, 0x400ff000, 0x40100000, True,  False, False, True,  c)
        check_region(12, 0x40100000, 0x40101000, True,  True,  False, True,  c)
        check_region(13, 0x40101000, 0x40122000, True,  False, True,  True,  d)
        check_region(14, 0x40122000, 0x40123000, True,  True,  True,  True,  d)
        check_region(15, 0x40123000, 0x40167000, True,  False, True,  True,  d)
        check_region(16, 0x40167000, 0x40169000, True,  False, False, True,  d)
        check_region(17, 0x40169000, 0x4016b000, True,  True,  False, True,  d)
        check_region(18, 0x4016b000, 0x40176000, True,  True,  False, True,  n)
        check_region(-1, 0x40176000, max_int,    False, False, False, False, n)

    @skipIfLLVMTargetMissing("X86")
    def test_minidump_sysroot(self):
        """Test that lldb can find a module referenced in an i386 linux minidump using the sysroot."""

        # Copy linux-x86_64 executable to tmp_sysroot/temp/test/ (since it was compiled as
        # /tmp/test/linux-x86_64)
        tmp_sysroot = os.path.join(
            self.getBuildDir(), "lldb_i386_mock_sysroot")
        executable = os.path.join(
            tmp_sysroot, "tmp", "test", "linux-x86_64")
        exe_dir = os.path.dirname(executable)
        lldbutil.mkdir_p(exe_dir)
        shutil.copyfile("linux-x86_64", executable)

        # Set sysroot and load core
        self.runCmd("platform select remote-linux --sysroot '%s'" %
                    tmp_sysroot)
        self.process_from_yaml("linux-x86_64.yaml")
        self.check_state()

        # Check that we loaded the module from the sysroot
        self.assertEqual(self.target.GetNumModules(), 1)
        module = self.target.GetModuleAtIndex(0)
        spec_dir_norm = os.path.normcase(module.GetFileSpec().GetDirectory())
        exe_dir_norm = os.path.normcase(exe_dir)
        self.assertEqual(spec_dir_norm, exe_dir_norm)




import os
from time import sleep

import gdbremote_testcase
import lldbgdbserverutils
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestGdbRemoteAttachOrWait(gdbremote_testcase.GdbRemoteTestCaseBase):

    mydir = TestBase.compute_mydir(__file__)

    @skipIfWindows # This test is flaky on Windows
    def test_launch_before_attach_with_vAttachOrWait(self):
        exe = '%s_%d' % (self.testMethodName, os.getpid())
        self.build(dictionary={'EXE': exe})
        self.set_inferior_startup_attach_manually()

        # Start the inferior, start the debug monitor, nothing is attached yet.
        procs = self.prep_debug_monitor_and_inferior(
            inferior_args=["sleep:60"],
            inferior_exe_path=self.getBuildArtifact(exe))
        self.assertIsNotNone(procs)

        # Make sure the target process has been launched.
        inferior = procs.get("inferior")
        self.assertIsNotNone(inferior)
        self.assertTrue(inferior.pid > 0)
        self.assertTrue(
            lldbgdbserverutils.process_is_running(
                inferior.pid, True))

        # Add attach packets.
        self.test_sequence.add_log_lines([
            # Do the attach.
            "read packet: $vAttachOrWait;{}#00".format(lldbgdbserverutils.gdbremote_hex_encode_string(exe)),
            # Expect a stop notification from the attach.
            {"direction": "send",
             "regex": r"^\$T([0-9a-fA-F]{2})[^#]*#[0-9a-fA-F]{2}$",
             "capture": {1: "stop_signal_hex"}},
        ], True)
        self.add_process_info_collection_packets()

        # Run the stream
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Gather process info response
        process_info = self.parse_process_info_response(context)
        self.assertIsNotNone(process_info)

        # Ensure the process id matches what we expected.
        pid_text = process_info.get('pid', None)
        self.assertIsNotNone(pid_text)
        reported_pid = int(pid_text, base=16)
        self.assertEqual(reported_pid, inferior.pid)

    @skipIfWindows # This test is flaky on Windows
    def test_launch_after_attach_with_vAttachOrWait(self):
        exe = '%s_%d' % (self.testMethodName, os.getpid())
        self.build(dictionary={'EXE': exe})
        self.set_inferior_startup_attach_manually()

        server = self.connect_to_debug_monitor()
        self.assertIsNotNone(server)

        self.do_handshake()
        self.test_sequence.add_log_lines([
            # Do the attach.
            "read packet: $vAttachOrWait;{}#00".format(lldbgdbserverutils.gdbremote_hex_encode_string(exe)),
        ], True)
        # Run the stream until attachWait.
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Sleep so we're sure that the inferior is launched after we ask for the attach.
        sleep(1)

        # Launch the inferior.
        inferior = self.launch_process_for_attach(
            inferior_args=["sleep:60"],
            exe_path=self.getBuildArtifact(exe))
        self.assertIsNotNone(inferior)
        self.assertTrue(inferior.pid > 0)
        self.assertTrue(
            lldbgdbserverutils.process_is_running(
                inferior.pid, True))

        # Make sure the attach succeeded.
        self.test_sequence.add_log_lines([
            {"direction": "send",
             "regex": r"^\$T([0-9a-fA-F]{2})[^#]*#[0-9a-fA-F]{2}$",
             "capture": {1: "stop_signal_hex"}},
        ], True)
        self.add_process_info_collection_packets()


        # Run the stream sending the response..
        context = self.expect_gdbremote_sequence()
        self.assertIsNotNone(context)

        # Gather process info response.
        process_info = self.parse_process_info_response(context)
        self.assertIsNotNone(process_info)

        # Ensure the process id matches what we expected.
        pid_text = process_info.get('pid', None)
        self.assertIsNotNone(pid_text)
        reported_pid = int(pid_text, base=16)
        self.assertEqual(reported_pid, inferior.pid)



# RUN: %PYTHON %s | FileCheck %s

import gc
from mlir.ir import *

def run(f):
  print("\nTEST:", f.__name__)
  f()
  gc.collect()
  assert Context._get_live_count() == 0
  return f


# CHECK-LABEL: TEST: testIntegerSetCapsule
@run
def testIntegerSetCapsule():
  with Context() as ctx:
    is1 = IntegerSet.get_empty(1, 1, ctx)
  capsule = is1._CAPIPtr
  # CHECK: mlir.ir.IntegerSet._CAPIPtr
  print(capsule)
  is2 = IntegerSet._CAPICreate(capsule)
  assert is1 == is2
  assert is2.context is ctx


# CHECK-LABEL: TEST: testIntegerSetGet
@run
def testIntegerSetGet():
  with Context():
    d0 = AffineDimExpr.get(0)
    d1 = AffineDimExpr.get(1)
    s0 = AffineSymbolExpr.get(0)
    c42 = AffineConstantExpr.get(42)

    # CHECK: (d0, d1)[s0] : (d0 - d1 == 0, s0 - 42 >= 0)
    set0 = IntegerSet.get(2, 1, [d0 - d1, s0 - c42], [True, False])
    print(set0)

    # CHECK: (d0)[s0] : (1 == 0)
    set1 = IntegerSet.get_empty(1, 1)
    print(set1)

    # CHECK: (d0)[s0, s1] : (d0 - s1 == 0, s0 - 42 >= 0)
    set2 = set0.get_replaced([d0, AffineSymbolExpr.get(1)], [s0], 1, 2)
    print(set2)

    try:
      IntegerSet.get(2, 1, [], [])
    except ValueError as e:
      # CHECK: Expected non-empty list of constraints
      print(e)

    try:
      IntegerSet.get(2, 1, [d0 - d1], [True, False])
    except ValueError as e:
      # CHECK: Expected the number of constraints to match that of equality flags
      print(e)

    try:
      IntegerSet.get(2, 1, [0], [True])
    except RuntimeError as e:
      # CHECK: Invalid expression when attempting to create an IntegerSet
      print(e)

    try:
      IntegerSet.get(2, 1, [None], [True])
    except RuntimeError as e:
      # CHECK: Invalid expression (None?) when attempting to create an IntegerSet
      print(e)

    try:
      set0.get_replaced([d0], [s0], 1, 1)
    except ValueError as e:
      # CHECK: Expected the number of dimension replacement expressions to match that of dimensions
      print(e)

    try:
      set0.get_replaced([d0, d1], [s0, s0], 1, 1)
    except ValueError as e:
      # CHECK: Expected the number of symbol replacement expressions to match that of symbols
      print(e)

    try:
      set0.get_replaced([d0, 1], [s0], 1, 1)
    except RuntimeError as e:
      # CHECK: Invalid expression when attempting to create an IntegerSet by replacing dimensions
      print(e)

    try:
      set0.get_replaced([d0, d1], [None], 1, 1)
    except RuntimeError as e:
      # CHECK: Invalid expression (None?) when attempting to create an IntegerSet by replacing symbols
      print(e)


# CHECK-LABEL: TEST: testIntegerSetProperties
@run
def testIntegerSetProperties():
  with Context():
    d0 = AffineDimExpr.get(0)
    d1 = AffineDimExpr.get(1)
    s0 = AffineSymbolExpr.get(0)
    c42 = AffineConstantExpr.get(42)

    set0 = IntegerSet.get(2, 1, [d0 - d1, s0 - c42, s0 - d0], [True, False, False])
    # CHECK: 2
    print(set0.n_dims)
    # CHECK: 1
    print(set0.n_symbols)
    # CHECK: 3
    print(set0.n_inputs)
    # CHECK: 1
    print(set0.n_equalities)
    # CHECK: 2
    print(set0.n_inequalities)

    # CHECK: 3
    print(len(set0.constraints))

    # CHECK-DAG: d0 - d1 == 0
    # CHECK-DAG: s0 - 42 >= 0
    # CHECK-DAG: -d0 + s0 >= 0
    for cstr in set0.constraints:
      print(cstr.expr, end='')
      print(" == 0" if cstr.is_eq else " >= 0")


# CHECK_LABEL: TEST: testHash
@run
def testHash():
  with Context():
    d0 = AffineDimExpr.get(0)
    d1 = AffineDimExpr.get(1)
    set = IntegerSet.get(2, 0, [d0 + d1], [True])

    assert hash(set) == hash(IntegerSet.get(2, 0, [d0 + d1], [True]))

    dictionary = dict()
    dictionary[set] = 42
    assert set in dictionary



"""
Test LLDB can set and hit watchpoints on tagged addresses
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class TestWatchTaggedAddresses(TestBase):

    mydir = TestBase.compute_mydir(__file__)
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # Skip this test if not running on AArch64 target that supports PAC
        if not self.isAArch64PAuth():
            self.skipTest('Target must support pointer authentication.')

        # Set source filename.
        self.source = 'main.c'

        # Invoke the default build rule.
        self.build()

        # Get the path of the executable
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(['linux']))
    def test_watch_hit_tagged_ptr_access(self):
        """
        Test that LLDB hits watchpoint installed on an untagged address with
        memory access by a tagged pointer.
        """

        # Add a breakpoint to set a watchpoint when stopped on the breakpoint.
        lldbutil.run_break_set_by_symbol(self, 'main')

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should be stopped due to the breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # Set the watchpoint variable declaration line number.
        self.decl = line_number(self.source,
                                '// Watchpoint variable declaration.')

        # Now let's set a watchpoint on 'global_var'.
        self.expect(
            "watchpoint set variable global_var",
            WATCHPOINT_CREATED,
            substrs=[
                'Watchpoint created',
                'size = 4',
                'type = w',
                '%s:%d' %
                (self.source,
                 self.decl)])

        self.verify_watch_hits()

    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(['linux']))
    def test_watch_set_on_tagged_ptr(self):
        """Test that LLDB can install and hit watchpoint on a tagged address"""

        # Find the line number to break inside main().
        self.line = line_number(self.source, '// Set break point at this line.')

        # Add a breakpoint to set a watchpoint when stopped on the breakpoint.
        lldbutil.run_break_set_by_file_and_line(
            self, None, self.line, num_expected_locations=1)

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should be stopped due to the breakpoint.
        self.expect("thread list", STOPPED_DUE_TO_BREAKPOINT,
                    substrs=['stopped',
                             'stop reason = breakpoint'])

        # Now let's set a expression watchpoint on 'tagged_ptr'.
        self.expect(
            "watchpoint set expression -s 4 -- tagged_ptr",
            WATCHPOINT_CREATED,
            substrs=[
                'Watchpoint created',
                'size = 4',
                'type = w'])

        self.verify_watch_hits()

    def verify_watch_hits(self):
        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect("watchpoint list -v",
                    substrs=['Number of supported hardware watchpoints:',
                             'hit_count = 0'])

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (read_write type).
        # The stop reason of the thread should be watchpoint.
        self.expect("thread backtrace", STOPPED_DUE_TO_WATCHPOINT,
                    substrs=['stop reason = watchpoint'])

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (read_write type).
        # The stop reason of the thread should be watchpoint.
        self.expect("thread backtrace", STOPPED_DUE_TO_WATCHPOINT,
                    substrs=['stop reason = watchpoint'])

        self.runCmd("process continue")

        # There should be no more watchpoint hit and the process status should
        # be 'exited'.
        self.expect("process status",
                    substrs=['exited'])

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should now be 2.
        self.expect("watchpoint list -v",
                    substrs=['hit_count = 2'])



"""
Test basic std::vector functionality.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBasicVector(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(self,
                                          "// Set break point at this line.",
                                          lldb.SBFileSpec("main.cpp"))

        self.runCmd("settings set target.import-std-module true")

        vector_type = "std::vector<int>"
        size_type = vector_type + "::size_type"
        iterator = vector_type + "::iterator"
        # LLDB's formatter provides us with a artificial 'item' member.
        iterator_children = [ValueCheck(name="item")]
        riterator = vector_type + "::reverse_iterator"
        riterator_children = [
            ValueCheck(name="__t"),
            ValueCheck(name="current")
        ]

        self.expect_expr("a",
                         result_type=vector_type,
                         result_children=[
                             ValueCheck(value="3"),
                             ValueCheck(value="1"),
                             ValueCheck(value="2")
                         ])
        self.expect_expr("a.size()", result_type=size_type, result_value="3")
        front = self.expect_expr("a.front()", result_value="3")
        value_type = front.GetDisplayTypeName()
        self.assertIn(value_type, [
            "std::vector<int>::value_type", # Pre-D112976
            "std::__vector_base<int, std::allocator<int> >::value_type", # Post-D112976
            ])
        self.expect_expr("a[1]", result_type=value_type, result_value="1")
        self.expect_expr("a.back()", result_type=value_type, result_value="2")

        self.expect("expr std::sort(a.begin(), a.end())")
        self.expect_expr("a.front()", result_type=value_type, result_value="1")
        self.expect_expr("a[1]", result_type=value_type, result_value="2")
        self.expect_expr("a.back()", result_type=value_type, result_value="3")

        self.expect("expr std::reverse(a.begin(), a.end())")
        self.expect_expr("a.front()", result_type=value_type, result_value="3")
        self.expect_expr("a[1]", result_type=value_type, result_value="2")
        self.expect_expr("a.back()", result_type=value_type, result_value="1")

        self.expect_expr("a.begin()",
                         result_type=iterator,
                         result_children=iterator_children)
        self.expect_expr("a.rbegin()",
                         result_type=riterator,
                         result_children=riterator_children)

        self.expect_expr("*a.begin()", result_type="int", result_value="3")
        self.expect_expr("*a.rbegin()", result_type="int", result_value="1")

        self.expect("expr a.pop_back()")
        self.expect_expr("a.back()", result_type=value_type, result_value="2")
        self.expect_expr("a.size()", result_type=size_type, result_value="2")

        self.expect_expr("a.at(0)", result_type=value_type, result_value="3")

        self.expect("expr a.push_back(4)")
        self.expect_expr("a.back()", result_type=value_type, result_value="4")
        self.expect_expr("a.size()", result_type=size_type, result_value="3")

        self.expect("expr a.emplace_back(5)")
        self.expect_expr("a.back()", result_type=value_type, result_value="5")
        self.expect_expr("a.size()", result_type=size_type, result_value="4")

        self.expect_expr("a",
                         result_children=[
                             ValueCheck(value="3"),
                             ValueCheck(value="2"),
                             ValueCheck(value="4"),
                             ValueCheck(value="5")
                         ])

        # Test that the typedef'd vector type can be substituted.
        self.expect("expr b.emplace_back(6)")
        self.expect_expr("b", result_type="vector_long",
                         result_children=[
                             ValueCheck(value="3"),
                             ValueCheck(value="1"),
                             ValueCheck(value="2"),
                             ValueCheck(value="6"),
                         ])



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import numpy as np
import mlir.dialects.builtin as builtin
import mlir.dialects.shape as shape


def run(f):
  print("\nTEST:", f.__name__)
  f()
  return f


# CHECK-LABEL: TEST: testConstShape
@run
def testConstShape():
  with Context() as ctx, Location.unknown():
    module = Module.create()
    f32 = F32Type.get()
    with InsertionPoint(module.body):
      @builtin.FuncOp.from_py_func(
          RankedTensorType.get((12, -1), f32))
      def const_shape_tensor(arg):
        return shape.ConstShapeOp(
          DenseElementsAttr.get(np.array([10, 20]), type=IndexType.get()))

    # CHECK-LABEL: func @const_shape_tensor(%arg0: tensor<12x?xf32>)
    # CHECK: shape.const_shape [10, 20] : tensor<2xindex>
    print(module)



# RUN: %PYTHON %s | FileCheck %s

from mlir_standalone.ir import *
from mlir_standalone.dialects import (
  builtin as builtin_d,
  standalone as standalone_d
)

with Context():
  standalone_d.register_dialect()
  module = Module.parse("""
    %0 = arith.constant 2 : i32
    %1 = standalone.foo %0 : i32
    """)
  # CHECK: %[[C:.*]] = arith.constant 2 : i32
  # CHECK: standalone.foo %[[C]] : i32
  print(str(module))



import lldb

num_hits = 0


def watchpoint_command(frame, wp, dict):
    global num_hits
    if num_hits == 0:
        print ("I stopped the first time")
        frame.EvaluateExpression("cookie = 888")
        num_hits += 1
        frame.thread.process.Continue()
    else:
        print ("I stopped the %d time" % (num_hits))
        frame.EvaluateExpression("cookie = 999")



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""Specializations for the Visual Studio 2015 interface."""

from dex.debugger.visualstudio.VisualStudio import VisualStudio


class VisualStudio2015(VisualStudio):
    @classmethod
    def get_name(cls):
        return 'Visual Studio 2015'

    @classmethod
    def get_option_name(cls):
        return 'vs2015'

    @property
    def _dte_version(self):
        return 'VisualStudio.DTE.14.0'




import unittest2

from lldbsuite.test.decorators import *
from lldbsuite.test.concurrent_base import ConcurrentEventsBase
from lldbsuite.test.lldbtest import TestBase


@skipIfWindows
class ConcurrentTwoBreakpointThreads(ConcurrentEventsBase):

    mydir = ConcurrentEventsBase.compute_mydir(__file__)

    # Atomic sequences are not supported yet for MIPS in LLDB.
    @skipIf(triple='^mips')
    @expectedFailureAll(archs=["aarch64"], oslist=["freebsd"],
                        bugnumber="llvm.org/pr49433")
    def test(self):
        """Test two threads that trigger a breakpoint. """
        self.build()
        self.do_thread_actions(num_breakpoint_threads=2)



"""
Test the 'gui' shortcuts 's','n','f','u','d' (step in, step over, step out, up, down)
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest

class TestGuiBasicDebugCommandTest(PExpectTest):

    mydir = TestBase.compute_mydir(__file__)

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIf(bugnumber="llvm.org/pr51833")
    @skipIfCursesSupportMissing
    def test_gui(self):
        self.build()

        self.launch(executable=self.getBuildArtifact("a.out"), dimensions=(100,500))
        self.expect('br set -f main.c -p "// Break here"', substrs=["Breakpoint 1", "address ="])
        self.expect("run", substrs=["stop reason ="])

        escape_key = chr(27).encode()

        # Start the GUI and close the welcome window.
        self.child.sendline("gui")
        self.child.expect("Welcome to the LLDB curses GUI.")
        self.child.send(escape_key)

        # Simulate a simple debugging session.
        self.child.send("s") # step
        self.child.expect("return 1; // In function[^\r\n]+<<< Thread 1: step in")
        self.child.send("u") # up
        self.child.expect_exact("func(); // Break here")
        self.child.send("d") # down
        self.child.expect_exact("return 1; // In function")
        self.child.send("f") # finish
        self.child.expect("<<< Thread 1: step out")
        self.child.send("s") # move onto the second one
        self.child.expect("<<< Thread 1: step in")
        self.child.send("n") # step over
        self.child.expect("<<< Thread 1: step over")

        # Press escape to quit the gui
        self.child.send(escape_key)

        self.expect_prompt()
        self.quit()



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""Utility functions related to DExTer's directory layout."""

import os


def get_root_directory():
    root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    assert os.path.basename(root) == 'dex', root
    return root



# RUN: %PYTHON -m mlir.dialects.linalg.opdsl.dump_oplib --file %s | FileCheck %s

from mlir.dialects.linalg.opdsl.lang import *


# CHECK: ---
# CHECK-LABEL: matmul
# CHECK: assignments:
# CHECK:  -
# CHECK:    arg: C
# CHECK:    value:
# CHECK:      arith_fn:
# CHECK:        fn_name: add
# CHECK:        operands:
# CHECK:          arith_fn:
# CHECK:            fn_name: mul
# CHECK:            operands:
# CHECK:              type_fn:
# CHECK:                type_var: U
# CHECK:                operands:
# CHECK:                  scalar_arg: A
# CHECK:              type_fn:
# CHECK:                type_var: U
# CHECK:                operands:
# CHECK:                  scalar_arg: B
@linalg_structured_op
def matmul(
    A=TensorDef(T, S.M, S.K),
    B=TensorDef(T, S.K, S.N),
    C=TensorDef(U, S.M, S.N, output=True)):
  C[D.m, D.n] += TypeFn.cast(U, A[D.m, D.k]) * TypeFn.cast(U, B[D.k, D.n])


# CHECK: ---
# CHECK-LABEL: constants
# CHECK: assignments:
# CHECK:  -
# CHECK:    arg: O
# CHECK:      arith_fn:
# CHECK:        fn_name: sub
# CHECK:        operands:
# CHECK:          arith_fn:
# CHECK:            fn_name: add
# CHECK:            operands:
# CHECK:              type_fn:
# CHECK:                type_var: T
# CHECK:                operands:
# CHECK:                  scalar_const: '3.1415926535897931 : f64'
# CHECK:              type_fn:
# CHECK:                type_var: T
# CHECK:                operands:
# CHECK:                  scalar_const: '42 : i64'
# CHECK:          type_fn:
# CHECK:            type_var: T
# CHECK:            operands:
# CHECK:              scalar_const: '1.{{[0]*}}e+03 : f64'
@linalg_structured_op
def constants(O=TensorDef(T, S.M, S.K, output=True)):
  pi = TypeFn.cast(T, const(3.1415926535897931))
  cst42 = TypeFn.cast(T, const(42))
  cst1000 = TypeFn.cast(T, const(1e+3))
  O[D.m, D.n] = pi + cst42 - cst1000


# CHECK: ---
# CHECK-LABEL: indices
# CHECK: assignments:
# CHECK:  -
# CHECK:    arg: O
# CHECK:      arith_fn:
# CHECK:        fn_name: add
# CHECK:        operands:
# CHECK:          scalar_index: 1
# CHECK:          scalar_index: 0
@linalg_structured_op
def indices(O=TensorDef(T, S.M, S.K, output=True)):
  O[D.m, D.n] = index(D.n) + index(D.m)


# CHECK: ---
# CHECK-LABEL: fill
# CHECK: assignments:
# CHECK:  -
# CHECK:    arg: O
# CHECK:      scalar_arg: value
@linalg_structured_op
def fill(value=ScalarDef(T), O=TensorDef(T, S.M, S.K, output=True)):
  O[D.m, D.n] = value



"""
Test basic std::shared_ptr functionality.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestSharedPtr(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(self,
                                          "// Set break point at this line.",
                                          lldb.SBFileSpec("main.cpp"))

        self.runCmd("settings set target.import-std-module true")

        self.expect_expr("s",
                         result_type="std::shared_ptr<int>",
                         result_summary="3 strong=1 weak=1",
                         result_children=[ValueCheck(name="__ptr_")])
        self.expect_expr("*s", result_type="int", result_value="3")
        self.expect_expr("*s = 5", result_type="int", result_value="5")
        self.expect_expr("*s", result_type="int", result_value="5")
        self.expect_expr("(bool)s", result_type="bool", result_value="true")
        self.expect("expr s.reset()")
        self.expect_expr("(bool)s", result_type="bool", result_value="false")



# RUN: %PYTHON %s | FileCheck %s

import gc
from mlir.ir import *

def run(f):
  print("\nTEST:", f.__name__)
  f()
  gc.collect()
  assert Context._get_live_count() == 0
  return f


# CHECK-LABEL: TEST: testAffineExprCapsule
@run
def testAffineExprCapsule():
  with Context() as ctx:
    affine_expr = AffineExpr.get_constant(42)

  affine_expr_capsule = affine_expr._CAPIPtr
  # CHECK: capsule object
  # CHECK: mlir.ir.AffineExpr._CAPIPtr
  print(affine_expr_capsule)

  affine_expr_2 = AffineExpr._CAPICreate(affine_expr_capsule)
  assert affine_expr == affine_expr_2
  assert affine_expr_2.context == ctx


# CHECK-LABEL: TEST: testAffineExprEq
@run
def testAffineExprEq():
  with Context():
    a1 = AffineExpr.get_constant(42)
    a2 = AffineExpr.get_constant(42)
    a3 = AffineExpr.get_constant(43)
    # CHECK: True
    print(a1 == a1)
    # CHECK: True
    print(a1 == a2)
    # CHECK: False
    print(a1 == a3)
    # CHECK: False
    print(a1 == None)
    # CHECK: False
    print(a1 == "foo")


# CHECK-LABEL: TEST: testAffineExprContext
@run
def testAffineExprContext():
  with Context():
    a1 = AffineExpr.get_constant(42)
  with Context():
    a2 = AffineExpr.get_constant(42)

  # CHECK: False
  print(a1 == a2)

run(testAffineExprContext)


# CHECK-LABEL: TEST: testAffineExprConstant
@run
def testAffineExprConstant():
  with Context():
    a1 = AffineExpr.get_constant(42)
    # CHECK: 42
    print(a1.value)
    # CHECK: 42
    print(a1)

    a2 = AffineConstantExpr.get(42)
    # CHECK: 42
    print(a2.value)
    # CHECK: 42
    print(a2)

    assert a1 == a2


# CHECK-LABEL: TEST: testAffineExprDim
@run
def testAffineExprDim():
  with Context():
    d1 = AffineExpr.get_dim(1)
    d11 = AffineDimExpr.get(1)
    d2 = AffineDimExpr.get(2)

    # CHECK: 1
    print(d1.position)
    # CHECK: d1
    print(d1)

    # CHECK: 2
    print(d2.position)
    # CHECK: d2
    print(d2)

    assert d1 == d11
    assert d1 != d2


# CHECK-LABEL: TEST: testAffineExprSymbol
@run
def testAffineExprSymbol():
  with Context():
    s1 = AffineExpr.get_symbol(1)
    s11 = AffineSymbolExpr.get(1)
    s2 = AffineSymbolExpr.get(2)

    # CHECK: 1
    print(s1.position)
    # CHECK: s1
    print(s1)

    # CHECK: 2
    print(s2.position)
    # CHEKC: s2
    print(s2)

    assert s1 == s11
    assert s1 != s2


# CHECK-LABEL: TEST: testAffineAddExpr
@run
def testAffineAddExpr():
  with Context():
    d1 = AffineDimExpr.get(1)
    d2 = AffineDimExpr.get(2)
    d12 = AffineExpr.get_add(d1, d2)
    # CHECK: d1 + d2
    print(d12)

    d12op = d1 + d2
    # CHECK: d1 + d2
    print(d12op)

    d1cst_op = d1 + 2
    # CHECK: d1 + 2
    print(d1cst_op)

    d1cst_op2 = 2 + d1
    # CHECK: d1 + 2
    print(d1cst_op2)

    assert d12 == d12op
    assert d12.lhs == d1
    assert d12.rhs == d2


# CHECK-LABEL: TEST: testAffineMulExpr
@run
def testAffineMulExpr():
  with Context():
    d1 = AffineDimExpr.get(1)
    c2 = AffineConstantExpr.get(2)
    expr = AffineExpr.get_mul(d1, c2)
    # CHECK: d1 * 2
    print(expr)

    # CHECK: d1 * 2
    op = d1 * c2
    print(op)

    # CHECK: d1 * 2
    op_cst = d1 * 2
    print(op_cst)

    # CHECK: d1 * 2
    op_cst2 = 2 * d1
    print(op_cst2)

    assert expr == op
    assert expr == op_cst
    assert expr.lhs == d1
    assert expr.rhs == c2


# CHECK-LABEL: TEST: testAffineModExpr
@run
def testAffineModExpr():
  with Context():
    d1 = AffineDimExpr.get(1)
    c2 = AffineConstantExpr.get(2)
    expr = AffineExpr.get_mod(d1, c2)
    # CHECK: d1 mod 2
    print(expr)

    # CHECK: d1 mod 2
    op = d1 % c2
    print(op)

    # CHECK: d1 mod 2
    op_cst = d1 % 2
    print(op_cst)

    # CHECK: 2 mod d1
    print(2 % d1)

    assert expr == op
    assert expr == op_cst
    assert expr.lhs == d1
    assert expr.rhs == c2

    expr2 = AffineExpr.get_mod(c2, d1)
    expr3 = AffineExpr.get_mod(2, d1)
    expr4 = AffineExpr.get_mod(d1, 2)

    # CHECK: 2 mod d1
    print(expr2)
    # CHECK: 2 mod d1
    print(expr3)
    # CHECK: d1 mod 2
    print(expr4)

    assert expr2 == expr3
    assert expr4 == expr


# CHECK-LABEL: TEST: testAffineFloorDivExpr
@run
def testAffineFloorDivExpr():
  with Context():
    d1 = AffineDimExpr.get(1)
    c2 = AffineConstantExpr.get(2)
    expr = AffineExpr.get_floor_div(d1, c2)
    # CHECK: d1 floordiv 2
    print(expr)

    assert expr.lhs == d1
    assert expr.rhs == c2

    expr2 = AffineExpr.get_floor_div(c2, d1)
    expr3 = AffineExpr.get_floor_div(2, d1)
    expr4 = AffineExpr.get_floor_div(d1, 2)

    # CHECK: 2 floordiv d1
    print(expr2)
    # CHECK: 2 floordiv d1
    print(expr3)
    # CHECK: d1 floordiv 2
    print(expr4)

    assert expr2 == expr3
    assert expr4 == expr


# CHECK-LABEL: TEST: testAffineCeilDivExpr
@run
def testAffineCeilDivExpr():
  with Context():
    d1 = AffineDimExpr.get(1)
    c2 = AffineConstantExpr.get(2)
    expr = AffineExpr.get_ceil_div(d1, c2)
    # CHECK: d1 ceildiv 2
    print(expr)

    assert expr.lhs == d1
    assert expr.rhs == c2

    expr2 = AffineExpr.get_ceil_div(c2, d1)
    expr3 = AffineExpr.get_ceil_div(2, d1)
    expr4 = AffineExpr.get_ceil_div(d1, 2)

    # CHECK: 2 ceildiv d1
    print(expr2)
    # CHECK: 2 ceildiv d1
    print(expr3)
    # CHECK: d1 ceildiv 2
    print(expr4)

    assert expr2 == expr3
    assert expr4 == expr


# CHECK-LABEL: TEST: testAffineExprSub
@run
def testAffineExprSub():
  with Context():
    d1 = AffineDimExpr.get(1)
    d2 = AffineDimExpr.get(2)
    expr = d1 - d2
    # CHECK: d1 - d2
    print(expr)

    assert expr.lhs == d1
    rhs = AffineMulExpr(expr.rhs)
    # CHECK: d2
    print(rhs.lhs)
    # CHECK: -1
    print(rhs.rhs)

    # CHECK: d1 - 42
    print(d1 - 42)
    # CHECK: -d1 + 42
    print(42 - d1)

    c42 = AffineConstantExpr.get(42)
    assert d1 - 42 == d1 - c42
    assert 42 - d1 == c42 - d1

# CHECK-LABEL: TEST: testClassHierarchy
@run
def testClassHierarchy():
  with Context():
    d1 = AffineDimExpr.get(1)
    c2 = AffineConstantExpr.get(2)
    add = AffineAddExpr.get(d1, c2)
    mul = AffineMulExpr.get(d1, c2)
    mod = AffineModExpr.get(d1, c2)
    floor_div = AffineFloorDivExpr.get(d1, c2)
    ceil_div = AffineCeilDivExpr.get(d1, c2)

    # CHECK: False
    print(isinstance(d1, AffineBinaryExpr))
    # CHECK: False
    print(isinstance(c2, AffineBinaryExpr))
    # CHECK: True
    print(isinstance(add, AffineBinaryExpr))
    # CHECK: True
    print(isinstance(mul, AffineBinaryExpr))
    # CHECK: True
    print(isinstance(mod, AffineBinaryExpr))
    # CHECK: True
    print(isinstance(floor_div, AffineBinaryExpr))
    # CHECK: True
    print(isinstance(ceil_div, AffineBinaryExpr))

    try:
      AffineBinaryExpr(d1)
    except ValueError as e:
      # CHECK: Cannot cast affine expression to AffineBinaryExpr
      print(e)

    try:
      AffineBinaryExpr(c2)
    except ValueError as e:
      # CHECK: Cannot cast affine expression to AffineBinaryExpr
      print(e)

# CHECK-LABEL: TEST: testIsInstance
@run
def testIsInstance():
  with Context():
    d1 = AffineDimExpr.get(1)
    c2 = AffineConstantExpr.get(2)
    add = AffineAddExpr.get(d1, c2)
    mul = AffineMulExpr.get(d1, c2)

    # CHECK: True
    print(AffineDimExpr.isinstance(d1))
    # CHECK: False
    print(AffineConstantExpr.isinstance(d1))
    # CHECK: True
    print(AffineConstantExpr.isinstance(c2))
    # CHECK: False
    print(AffineMulExpr.isinstance(c2))
    # CHECK: True
    print(AffineAddExpr.isinstance(add))
    # CHECK: False
    print(AffineMulExpr.isinstance(add))
    # CHECK: True
    print(AffineMulExpr.isinstance(mul))
    # CHECK: False
    print(AffineAddExpr.isinstance(mul))


# CHECK-LABEL: TEST: testCompose
@run
def testCompose():
  with Context():
    # d0 + d2.
    expr = AffineAddExpr.get(AffineDimExpr.get(0), AffineDimExpr.get(2))

    # (d0, d1, d2)[s0, s1] -> (d0 + s1, d1 + s0, d0 + d1 + d2)
    map1 = AffineAddExpr.get(AffineDimExpr.get(0), AffineSymbolExpr.get(1))
    map2 = AffineAddExpr.get(AffineDimExpr.get(1), AffineSymbolExpr.get(0))
    map3 = AffineAddExpr.get(
        AffineAddExpr.get(AffineDimExpr.get(0), AffineDimExpr.get(1)),
        AffineDimExpr.get(2))
    map = AffineMap.get(3, 2, [map1, map2, map3])

    # CHECK: d0 + s1 + d0 + d1 + d2
    print(expr.compose(map))


# CHECK-LABEL: TEST: testHash
@run
def testHash():
  with Context():
    d0 = AffineDimExpr.get(0)
    s1 = AffineSymbolExpr.get(1)
    assert hash(d0) == hash(AffineDimExpr.get(0))
    assert hash(d0 + s1) == hash(AffineAddExpr.get(d0, s1))

    dictionary = dict()
    dictionary[d0] = 0
    dictionary[s1] = 1
    assert d0 in dictionary
    assert s1 in dictionary



# DExTer : Debugging Experience Tester
# ~~~~~~   ~         ~~         ~   ~~
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

import os

from .utils import *

class Frame(object):
  def __init__(self, frame, idx, Symbols):
    # Store some base information about the frame
    self.ip = frame.InstructionOffset
    self.scope_idx = idx
    self.virtual = frame.Virtual
    self.inline_frame_context = frame.InlineFrameContext
    self.func_tbl_entry = frame.FuncTableEntry

    # Fetch the module/symbol we're in, with displacement. Useful for debugging.
    self.descr = Symbols.GetNearNameByOffset(self.ip)
    split = self.descr.split('!')[0]
    self.module = split[0]
    self.symbol = split[1]

    # Fetch symbol group for this scope.
    prevscope = Symbols.GetCurrentScopeFrameIndex()
    if Symbols.SetScopeFrameByIndex(idx):
      symgroup = Symbols.GetScopeSymbolGroup2()
      Symbols.SetScopeFrameByIndex(prevscope)
      self.symgroup = symgroup
    else:
      self.symgroup = None

    # Fetch the name according to the line-table, using inlining context.
    name = Symbols.GetNameByInlineContext(self.ip, self.inline_frame_context)
    self.function_name = name.split('!')[-1]

    try:
      tup = Symbols.GetLineByInlineContext(self.ip, self.inline_frame_context)
      self.source_file, self.line_no = tup
    except WinError as e:
      # Fall back to trying to use a non-inlining-aware line number
      # XXX - this is not inlining aware
      sym = Symbols.GetLineByOffset(self.ip)
      if sym is not None:
        self.source_file, self.line_no = sym
      else:
        self.source_file = None
        self.line_no = None
        self.basename = None

    if self.source_file is not None:
      self.basename = os.path.basename(self.source_file)
    else:
      self.basename = None



  def __str__(self):
    return '{}:{}({}) {}'.format(self.basename, self.line, self.descr, self.function_name)

def main_on_stack(Symbols, frames):
  module_name = Symbols.get_exefile_module_name()
  main_name = "{}!main".format(module_name)
  for x in frames:
    if main_name in x.descr: # Could be less hard coded...
      return True
  return False

def probe_state(Client):
  # Fetch the state of the program -- represented by the stack frames.
  frames, numframes = Client.Control.GetStackTraceEx()

  the_frames = [Frame(frames[x], x, Client.Symbols) for x in range(numframes)]
  if not main_on_stack(Client.Symbols, the_frames):
    return None

  return the_frames



"""
This is a test to ensure that both lldb is reconstructing the right
calling convention for a CXXRecordDecl as represented by:

   DW_CC_pass_by_reference
   DW_CC_pass_by_value

and to also make sure that the ASTImporter is copying over this
setting when importing the CXXRecordDecl via setArgPassingRestrictions.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestArgumentPassingRestrictions(TestBase):

  mydir = TestBase.compute_mydir(__file__)

  @skipIf(compiler="clang", compiler_version=['<', '7.0'])
  def test_argument_passing_restrictions(self):
    self.build()

    lldbutil.run_to_source_breakpoint(self, '// break here',
            lldb.SBFileSpec("main.cpp"))

    self.expect_expr("returnPassByRef()", result_type="PassByRef", result_children=[
        ValueCheck(name="x", value="11223344")
    ])

    self.expect_expr("takePassByRef(p)", result_type="int", result_value="42")



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""YAML serialization is routed through here to centralize common logic."""

import sys

try:
  import yaml
except ModuleNotFoundError as e:
  raise ModuleNotFoundError(
      f"This tool requires PyYAML but it was not installed. "
      f"Recommend: {sys.executable} -m pip install PyYAML") from e

__all__ = [
    "yaml_dump",
    "yaml_dump_all",
    "YAMLObject",
]


class YAMLObject(yaml.YAMLObject):

  @classmethod
  def to_yaml(cls, dumper, self):
    """Default to a custom dictionary mapping."""
    return dumper.represent_mapping(cls.yaml_tag, self.to_yaml_custom_dict())

  def to_yaml_custom_dict(self):
    raise NotImplementedError()

  def as_linalg_yaml(self):
    return yaml_dump(self)


def multiline_str_representer(dumper, data):
  if len(data.splitlines()) > 1:
    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='|')
  else:
    return dumper.represent_scalar('tag:yaml.org,2002:str', data)


yaml.add_representer(str, multiline_str_representer)


def yaml_dump(data, sort_keys=False, **kwargs):
  return yaml.dump(data, sort_keys=sort_keys, **kwargs)


def yaml_dump_all(data, sort_keys=False, explicit_start=True, **kwargs):
  return yaml.dump_all(data,
                       sort_keys=sort_keys,
                       explicit_start=explicit_start,
                       **kwargs)



"""Test calling functions in static methods."""



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestObjCStaticMethod(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break inside main().
        self.main_source = "static.m"
        self.break_line = line_number(
            self.main_source, '// Set breakpoint here.')

    @add_test_categories(['pyapi'])
    #<rdar://problem/9745789> "expression" can't call functions in class methods
    def test_with_python_api(self):
        """Test calling functions in static methods."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        bpt = target.BreakpointCreateByLocation(
            self.main_source, self.break_line)
        self.assertTrue(bpt, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(
            None, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        thread_list = lldbutil.get_threads_stopped_at_breakpoint(process, bpt)

        # Make sure we stopped at the first breakpoint.
        self.assertNotEqual(
            len(thread_list), 0,
            "No thread stopped at our breakpoint.")
        self.assertEquals(len(thread_list), 1,
                        "More than one thread stopped at our breakpoint.")

        # Now make sure we can call a function in the static method we've
        # stopped in.
        frame = thread_list[0].GetFrameAtIndex(0)
        self.assertTrue(frame, "Got a valid frame 0 frame.")

        cmd_value = frame.EvaluateExpression("(char *) sel_getName (_cmd)")
        self.assertTrue(cmd_value.IsValid())
        sel_name = cmd_value.GetSummary()
        self.assertEqual(
            sel_name, "\"doSomethingWithString:\"",
            "Got the right value for the selector as string.")

        cmd_value = frame.EvaluateExpression(
            "[self doSomethingElseWithString:string]")
        self.assertTrue(cmd_value.IsValid())
        string_length = cmd_value.GetValueAsUnsigned()
        self.assertEqual(
            string_length, 27,
            "Got the right value from another class method on the same class.")



"""
Test lldb-vscode coreFile attaching
"""


import unittest2
import vscode
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbvscode_testcase
import os


class TestVSCode_coreFile(lldbvscode_testcase.VSCodeTestCaseBase):

    mydir = TestBase.compute_mydir(__file__)

    @skipIfWindows
    @skipIfRemote
    @skipIfLLVMTargetMissing("X86")
    def test_core_file(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        exe_file = os.path.join(current_dir, "linux-x86_64.out")
        core_file = os.path.join(current_dir, "linux-x86_64.core")

        self.create_debug_adaptor()
        self.attach(exe_file, coreFile=core_file)

        expected_frames = [
            {'column': 0, 'id': 524288, 'line': 4, 'name': 'bar', 'source': {'name': 'main.c', 'path': '/home/labath/test/main.c'}},
            {'column': 0, 'id': 524289, 'line': 10, 'name': 'foo', 'source': {'name': 'main.c', 'path': '/home/labath/test/main.c'}},
            {'column': 0, 'id': 524290, 'line': 16, 'name': '_start', 'source': {'name': 'main.c', 'path': '/home/labath/test/main.c'}},
        ]

        self.assertEquals(self.get_stackFrames(), expected_frames)

        # Resuming should have no effect and keep the process stopped
        self.continue_to_next_stop()
        self.assertEquals(self.get_stackFrames(), expected_frames)

        self.vscode.request_next(threadId=32259)
        self.assertEquals(self.get_stackFrames(), expected_frames)

    @skipIfWindows
    @skipIfRemote
    @skipIfLLVMTargetMissing("X86")
    def test_core_file_source_mapping(self):
        ''' Test that sourceMap property is correctly applied when loading a core '''
        current_dir = os.path.dirname(os.path.realpath(__file__))
        exe_file = os.path.join(current_dir, "linux-x86_64.out")
        core_file = os.path.join(current_dir, "linux-x86_64.core")

        self.create_debug_adaptor()

        source_map = [["/home/labath/test", current_dir]]
        self.attach(exe_file, coreFile=core_file, sourceMap=source_map)

        self.assertTrue(current_dir in self.get_stackFrames()[0]['source']['path'])



#!/usr/bin/env python
#===-- x86_64_target_definition.py -----------------------------*- C++ -*-===//
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
#===----------------------------------------------------------------------===//

#----------------------------------------------------------------------
# DESCRIPTION
#
# This file can be used with the following setting:
#   plugin.process.gdb-remote.target-definition-file
# This setting should be used when you are trying to connect to a
# remote GDB server that doesn't support any of the register discovery
# packets that LLDB normally uses.
#
# Why is this necessary? LLDB doesn't require a new build of LLDB that
# targets each new architecture you will debug with. Instead, all
# architectures are supported and LLDB relies on extra GDB server
# packets to discover the target we are connecting to so that is can
# show the right registers for each target. This allows the GDB server
# to change and add new registers without requiring a new LLDB build
# just so we can see new registers.
#
# This file implements the x86_64 registers for the darwin version of
# GDB and allows you to connect to servers that use this register set.
#
# USAGE
#
# (lldb) settings set plugin.process.gdb-remote.target-definition-file /path/to/x86_64_target_definition.py
# (lldb) gdb-remote other.baz.com:1234
#
# The target definition file will get used if and only if the
# qRegisterInfo packets are not supported when connecting to a remote
# GDB server.
#----------------------------------------------------------------------
from lldb import *

# Compiler and DWARF register numbers
name_to_gcc_dwarf_regnum = {
    'rax': 0,
    'rdx': 1,
    'rcx': 2,
    'rbx': 3,
    'rsi': 4,
    'rdi': 5,
    'rbp': 6,
    'rsp': 7,
    'r8': 8,
    'r9': 9,
    'r10': 10,
    'r11': 11,
    'r12': 12,
    'r13': 13,
    'r14': 14,
    'r15': 15,
    'rip': 16,
    'xmm0': 17,
    'xmm1': 18,
    'xmm2': 19,
    'xmm3': 20,
    'xmm4': 21,
    'xmm5': 22,
    'xmm6': 23,
    'xmm7': 24,
    'xmm8': 25,
    'xmm9': 26,
    'xmm10': 27,
    'xmm11': 28,
    'xmm12': 29,
    'xmm13': 30,
    'xmm14': 31,
    'xmm15': 32,
    'stmm0': 33,
    'stmm1': 34,
    'stmm2': 35,
    'stmm3': 36,
    'stmm4': 37,
    'stmm5': 38,
    'stmm6': 39,
    'stmm7': 30,
    'ymm0': 41,
    'ymm1': 42,
    'ymm2': 43,
    'ymm3': 44,
    'ymm4': 45,
    'ymm5': 46,
    'ymm6': 47,
    'ymm7': 48,
    'ymm8': 49,
    'ymm9': 40,
    'ymm10': 41,
    'ymm11': 42,
    'ymm12': 43,
    'ymm13': 44,
    'ymm14': 45,
    'ymm15': 46
}

name_to_gdb_regnum = {
    'rax': 0,
    'rbx': 1,
    'rcx': 2,
    'rdx': 3,
    'rsi': 4,
    'rdi': 5,
    'rbp': 6,
    'rsp': 7,
    'r8': 8,
    'r9': 9,
    'r10': 10,
    'r11': 11,
    'r12': 12,
    'r13': 13,
    'r14': 14,
    'r15': 15,
    'rip': 16,
    'rflags': 17,
    'cs': 18,
    'ss': 19,
    'ds': 20,
    'es': 21,
    'fs': 22,
    'gs': 23,
    'stmm0': 24,
    'stmm1': 25,
    'stmm2': 26,
    'stmm3': 27,
    'stmm4': 28,
    'stmm5': 29,
    'stmm6': 30,
    'stmm7': 31,
    'fctrl': 32,
    'fstat': 33,
    'ftag': 34,
    'fiseg': 35,
    'fioff': 36,
    'foseg': 37,
    'fooff': 38,
    'fop': 39,
    'xmm0': 40,
    'xmm1': 41,
    'xmm2': 42,
    'xmm3': 43,
    'xmm4': 44,
    'xmm5': 45,
    'xmm6': 46,
    'xmm7': 47,
    'xmm8': 48,
    'xmm9': 49,
    'xmm10': 50,
    'xmm11': 51,
    'xmm12': 52,
    'xmm13': 53,
    'xmm14': 54,
    'xmm15': 55,
    'mxcsr': 56,
    'ymm0': 57,
    'ymm1': 58,
    'ymm2': 59,
    'ymm3': 60,
    'ymm4': 61,
    'ymm5': 62,
    'ymm6': 63,
    'ymm7': 64,
    'ymm8': 65,
    'ymm9': 66,
    'ymm10': 67,
    'ymm11': 68,
    'ymm12': 69,
    'ymm13': 70,
    'ymm14': 71,
    'ymm15': 72
}

name_to_generic_regnum = {
    'rip': LLDB_REGNUM_GENERIC_PC,
    'rsp': LLDB_REGNUM_GENERIC_SP,
    'rbp': LLDB_REGNUM_GENERIC_FP,
    'rdi': LLDB_REGNUM_GENERIC_ARG1,
    'rsi': LLDB_REGNUM_GENERIC_ARG2,
    'rdx': LLDB_REGNUM_GENERIC_ARG3,
    'rcx': LLDB_REGNUM_GENERIC_ARG4,
    'r8': LLDB_REGNUM_GENERIC_ARG5,
    'r9': LLDB_REGNUM_GENERIC_ARG6
}


def get_reg_num(reg_num_dict, reg_name):
    if reg_name in reg_num_dict:
        return reg_num_dict[reg_name]
    return LLDB_INVALID_REGNUM


def get_reg_num(reg_num_dict, reg_name):
    if reg_name in reg_num_dict:
        return reg_num_dict[reg_name]
    return LLDB_INVALID_REGNUM

x86_64_register_infos = [
    {'name': 'rax',
     'set': 0,
     'bitsize': 64,
     'encoding': eEncodingUint,
     'format': eFormatAddressInfo},
    {'name': 'rbx',
     'set': 0,
     'bitsize': 64,
     'encoding': eEncodingUint,
     'format': eFormatAddressInfo},
    {'name': 'rcx', 'set': 0, 'bitsize': 64, 'encoding': eEncodingUint,
        'format': eFormatAddressInfo, 'alt-name': 'arg4'},
    {'name': 'rdx', 'set': 0, 'bitsize': 64, 'encoding': eEncodingUint,
        'format': eFormatAddressInfo, 'alt-name': 'arg3'},
    {'name': 'rsi', 'set': 0, 'bitsize': 64, 'encoding': eEncodingUint,
        'format': eFormatAddressInfo, 'alt-name': 'arg2'},
    {'name': 'rdi', 'set': 0, 'bitsize': 64, 'encoding': eEncodingUint,
        'format': eFormatAddressInfo, 'alt-name': 'arg1'},
    {'name': 'rbp', 'set': 0, 'bitsize': 64, 'encoding': eEncodingUint,
        'format': eFormatAddressInfo, 'alt-name': 'fp'},
    {'name': 'rsp', 'set': 0, 'bitsize': 64, 'encoding': eEncodingUint,
        'format': eFormatAddressInfo, 'alt-name': 'sp'},
    {'name': 'r8', 'set': 0, 'bitsize': 64, 'encoding': eEncodingUint,
        'format': eFormatAddressInfo, 'alt-name': 'arg5'},
    {'name': 'r9', 'set': 0, 'bitsize': 64, 'encoding': eEncodingUint,
        'format': eFormatAddressInfo, 'alt-name': 'arg6'},
    {'name': 'r10',
     'set': 0,
     'bitsize': 64,
     'encoding': eEncodingUint,
     'format': eFormatAddressInfo},
    {'name': 'r11',
     'set': 0,
     'bitsize': 64,
     'encoding': eEncodingUint,
     'format': eFormatAddressInfo},
    {'name': 'r12',
     'set': 0,
     'bitsize': 64,
     'encoding': eEncodingUint,
     'format': eFormatAddressInfo},
    {'name': 'r13',
     'set': 0,
     'bitsize': 64,
     'encoding': eEncodingUint,
     'format': eFormatAddressInfo},
    {'name': 'r14',
     'set': 0,
     'bitsize': 64,
     'encoding': eEncodingUint,
     'format': eFormatAddressInfo},
    {'name': 'r15',
     'set': 0,
     'bitsize': 64,
     'encoding': eEncodingUint,
     'format': eFormatAddressInfo},
    {'name': 'rip', 'set': 0, 'bitsize': 64, 'encoding': eEncodingUint,
        'format': eFormatAddressInfo, 'alt-name': 'pc'},
    {'name': 'rflags', 'set': 0, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'cs', 'set': 0, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'ss', 'set': 0, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'ds', 'set': 0, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'es', 'set': 0, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'fs', 'set': 0, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'gs', 'set': 0, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'stmm0',
     'set': 1,
     'bitsize': 80,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'stmm1',
     'set': 1,
     'bitsize': 80,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'stmm2',
     'set': 1,
     'bitsize': 80,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'stmm3',
     'set': 1,
     'bitsize': 80,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'stmm4',
     'set': 1,
     'bitsize': 80,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'stmm5',
     'set': 1,
     'bitsize': 80,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'stmm6',
     'set': 1,
     'bitsize': 80,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'stmm7',
     'set': 1,
     'bitsize': 80,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'fctrl', 'set': 1, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'fstat', 'set': 1, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'ftag', 'set': 1, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'fiseg', 'set': 1, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'fioff', 'set': 1, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'foseg', 'set': 1, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'fooff', 'set': 1, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'fop', 'set': 1, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    {'name': 'xmm0',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm1',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm2',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm3',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm4',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm5',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm6',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm7',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm8',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm9',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm10',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm11',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm12',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm13',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm14',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'xmm15',
     'set': 1,
     'bitsize': 128,
     'encoding': eEncodingVector,
     'format': eFormatVectorOfUInt8},
    {'name': 'mxcsr', 'set': 1, 'bitsize': 32,
        'encoding': eEncodingUint, 'format': eFormatHex},
    # Registers that are contained in or composed of one of more other
    # registers
    {'name': 'eax',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rax[31:0]'},
    {'name': 'ebx',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rbx[31:0]'},
    {'name': 'ecx',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rcx[31:0]'},
    {'name': 'edx',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rdx[31:0]'},
    {'name': 'edi',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rdi[31:0]'},
    {'name': 'esi',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rsi[31:0]'},
    {'name': 'ebp',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rbp[31:0]'},
    {'name': 'esp',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rsp[31:0]'},
    {'name': 'r8d',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r8[31:0]'},
    {'name': 'r9d',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r9[31:0]'},
    {'name': 'r10d',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r10[31:0]'},
    {'name': 'r11d',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r11[31:0]'},
    {'name': 'r12d',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r12[31:0]'},
    {'name': 'r13d',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r13[31:0]'},
    {'name': 'r14d',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r14[31:0]'},
    {'name': 'r15d',
     'set': 0,
     'bitsize': 32,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r15[31:0]'},

    {'name': 'ax',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rax[15:0]'},
    {'name': 'bx',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rbx[15:0]'},
    {'name': 'cx',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rcx[15:0]'},
    {'name': 'dx',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rdx[15:0]'},
    {'name': 'di',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rdi[15:0]'},
    {'name': 'si',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rsi[15:0]'},
    {'name': 'bp',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rbp[15:0]'},
    {'name': 'sp',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rsp[15:0]'},
    {'name': 'r8w',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r8[15:0]'},
    {'name': 'r9w',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r9[15:0]'},
    {'name': 'r10w',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r10[15:0]'},
    {'name': 'r11w',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r11[15:0]'},
    {'name': 'r12w',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r12[15:0]'},
    {'name': 'r13w',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r13[15:0]'},
    {'name': 'r14w',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r14[15:0]'},
    {'name': 'r15w',
     'set': 0,
     'bitsize': 16,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r15[15:0]'},

    {'name': 'ah',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rax[15:8]'},
    {'name': 'bh',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rbx[15:8]'},
    {'name': 'ch',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rcx[15:8]'},
    {'name': 'dh',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rdx[15:8]'},

    {'name': 'al',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rax[7:0]'},
    {'name': 'bl',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rbx[7:0]'},
    {'name': 'cl',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rcx[7:0]'},
    {'name': 'dl',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rdx[7:0]'},
    {'name': 'dil',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rdi[7:0]'},
    {'name': 'sil',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rsi[7:0]'},
    {'name': 'bpl',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rbp[7:0]'},
    {'name': 'spl',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'rsp[7:0]'},
    {'name': 'r8l',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r8[7:0]'},
    {'name': 'r9l',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r9[7:0]'},
    {'name': 'r10l',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r10[7:0]'},
    {'name': 'r11l',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r11[7:0]'},
    {'name': 'r12l',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r12[7:0]'},
    {'name': 'r13l',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r13[7:0]'},
    {'name': 'r14l',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r14[7:0]'},
    {'name': 'r15l',
     'set': 0,
     'bitsize': 8,
     'encoding': eEncodingUint,
     'format': eFormatHex,
     'slice': 'r15[7:0]'},
]

g_target_definition = None


def get_target_definition():
    global g_target_definition
    if g_target_definition is None:
        g_target_definition = {}
        offset = 0
        for reg_info in x86_64_register_infos:
            reg_name = reg_info['name']

            # Only fill in the offset if there is no 'slice' in the register
            # info
            if 'slice' not in reg_info and 'composite' not in reg_info:
                reg_info['offset'] = offset
                offset += reg_info['bitsize'] // 8

            # Set the GCC/DWARF register number for this register if it has one
            reg_num = get_reg_num(name_to_gcc_dwarf_regnum, reg_name)
            if reg_num != LLDB_INVALID_REGNUM:
                reg_info['gcc'] = reg_num
                reg_info['dwarf'] = reg_num

            # Set the generic register number for this register if it has one
            reg_num = get_reg_num(name_to_generic_regnum, reg_name)
            if reg_num != LLDB_INVALID_REGNUM:
                reg_info['generic'] = reg_num

            # Set the GDB register number for this register if it has one
            reg_num = get_reg_num(name_to_gdb_regnum, reg_name)
            if reg_num != LLDB_INVALID_REGNUM:
                reg_info['gdb'] = reg_num

        g_target_definition['sets'] = [
            'General Purpose Registers',
            'Floating Point Registers']
        g_target_definition['registers'] = x86_64_register_infos
        g_target_definition[
            'host-info'] = {'triple': 'x86_64-apple-macosx', 'endian': eByteOrderLittle}
        g_target_definition['g-packet-size'] = offset
    return g_target_definition


def get_dynamic_setting(target, setting_name):
    if setting_name == 'gdb-server-target-definition':
        return get_target_definition()



"""GDB pretty printers for MLIR types."""

import gdb.printing


class StoragePrinter:
  """Prints bases of a struct and its fields."""

  def __init__(self, val):
    self.val = val

  def children(self):
    for field in self.val.type.fields():
      if field.is_base_class:
        yield '<%s>' % field.name, self.val.cast(field.type)
      else:
        yield field.name, self.val[field.name]


class TupleTypeStoragePrinter(StoragePrinter):

  def children(self):
    for child in StoragePrinter.children(self):
      yield child
    pointer_type = gdb.lookup_type('mlir::Type').pointer()
    elements = (self.val.address + 1).cast(pointer_type)
    for i in range(self.val['numElements']):
      yield 'elements[%u]' % i, elements[i]


class FusedLocationStoragePrinter(StoragePrinter):

  def children(self):
    for child in StoragePrinter.children(self):
      yield child
    pointer_type = gdb.lookup_type('mlir::Location').pointer()
    elements = (self.val.address + 1).cast(pointer_type)
    for i in range(self.val['numLocs']):
      yield 'locs[%u]' % i, elements[i]


class StorageTypeMap:
  """Maps a TypeID to the corresponding concrete type.

  Types need to be registered by name before the first lookup.
  """

  def __init__(self):
    self.map = None
    self.type_names = []

  def register_type(self, type_name):
    assert not self.map, 'register_type called after __getitem__'
    self.type_names += [type_name]

  def _init_map(self):
    """Lazy initialization  of self.map."""
    if self.map:
      return
    self.map = {}
    for type_name in self.type_names:
      concrete_type = gdb.lookup_type(type_name)
      try:
        storage = gdb.parse_and_eval(
            "&'mlir::detail::TypeIDExported::get<%s>()::instance'" % type_name)
      except gdb.error:
        # Skip when TypeID instance cannot be found in current context.
        continue
      if concrete_type and storage:
        self.map[int(storage)] = concrete_type

  def __getitem__(self, type_id):
    self._init_map()
    return self.map.get(int(type_id['storage']))


storage_type_map = StorageTypeMap()


def get_type_id_printer(val):
  """Returns a printer of the name of a mlir::TypeID."""

  class TypeIdPrinter:

    def __init__(self, string):
      self.string = string

    def to_string(self):
      return self.string

  concrete_type = storage_type_map[val]
  if not concrete_type:
    return None
  return TypeIdPrinter('mlir::TypeID::get<%s>()' % concrete_type)


def get_attr_or_type_printer(val, get_type_id):
  """Returns a printer for mlir::Attribute or mlir::Type."""

  class AttrOrTypePrinter:

    def __init__(self, type_id, impl):
      self.type_id = type_id
      self.impl = impl

    def children(self):
      yield 'typeID', self.type_id
      yield 'cast<%s>(impl)' % self.impl.type, self.impl

  if not val['impl']:
    return None
  impl = val['impl'].dereference()
  type_id = get_type_id(impl)
  concrete_type = storage_type_map[type_id]
  if not concrete_type:
    return None
  # 3rd template argument of StorageUserBase is the storage type.
  storage_type = concrete_type.fields()[0].type.template_argument(2)
  if not storage_type:
    return None
  return AttrOrTypePrinter(type_id, impl.cast(storage_type))


class ImplPrinter:
  """Printer for an instance with a single 'impl' member pointer."""

  def __init__(self, val):
    self.impl = val['impl']

  def children(self):
    yield 'impl', (self.impl.dereference() if self.impl else self.impl)


# Printers of types deriving from Attribute::AttrBase or Type::TypeBase.
for name in [
    # mlir/IR/Attributes.h
    'ArrayAttr',
    'DictionaryAttr',
    'FloatAttr',
    'IntegerAttr',
    'IntegerSetAttr',
    'OpaqueAttr',
    'StringAttr',
    'SymbolRefAttr',
    'TypeAttr',
    'UnitAttr',
    'DenseStringElementsAttr',
    'DenseIntOrFPElementsAttr',
    'OpaqueElementsAttr',
    'SparseElementsAttr',
    # mlir/IR/BuiltinTypes.h
    'ComplexType',
    'IndexType',
    'IntegerType',
    'Float16Type',
    'Float32Type',
    'Float64Type',
    'Float80Type',
    'Float128Type',
    'NoneType',
    'VectorType',
    'RankedTensorType',
    'UnrankedTensorType',
    'MemRefType',
    'UnrankedMemRefType',
    'TupleType',
    # mlir/IR/Location.h
    'CallSiteLoc',
    'FileLineColLoc',
    'FusedLoc',
    'NameLoc',
    'OpaqueLoc',
    'UnknownLoc'
]:
  storage_type_map.register_type('mlir::%s' % name)  # Register for upcasting.
storage_type_map.register_type('void')  # Register default.


pp = gdb.printing.RegexpCollectionPrettyPrinter('MLIRSupport')

pp.add_printer('mlir::OperationName', '^mlir::OperationName$', ImplPrinter)
pp.add_printer('mlir::Value', '^mlir::Value$', ImplPrinter)

# Printers for types deriving from AttributeStorage or TypeStorage.
pp.add_printer('mlir::detail::FusedLocationStorage',
               '^mlir::detail::FusedLocationStorage',
               FusedLocationStoragePrinter)
pp.add_printer('mlir::detail::TupleTypeStorage',
               '^mlir::detail::TupleTypeStorage$', TupleTypeStoragePrinter)

pp.add_printer('mlir::TypeID', '^mlir::TypeID$', get_type_id_printer)


def add_attr_or_type_printers(name):
  """Adds printers for mlir::Attribute or mlir::Type and their Storage type."""
  get_type_id = lambda val: val['abstract%s' % name]['typeID']
  pp.add_printer('mlir::%s' % name, '^mlir::%s$' % name,
                 lambda val: get_attr_or_type_printer(val, get_type_id))


# Upcasting printers of mlir::Attribute and mlir::Type.
for name in ['Attribute', 'Type']:
  add_attr_or_type_printers(name)

gdb.printing.register_pretty_printer(gdb.current_objfile(), pp)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

class TestCase(TestBase):

    mydir = TestBase.compute_mydir(__file__)

    def common_setup(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "// break here", lldb.SBFileSpec("main.cpp"))

    def test_call_on_base(self):
        self.common_setup()
        self.expect_expr("base_with_dtor.foo()", result_type="int", result_value="1")
        self.expect_expr("base_without_dtor.foo()", result_type="int", result_value="2")

    def test_call_on_derived(self):
        self.common_setup()
        self.expect_expr("derived_with_dtor.foo()", result_type="int", result_value="3")
        self.expect_expr("derived_without_dtor.foo()", result_type="int", result_value="4")
        self.expect_expr("derived_with_base_dtor.foo()", result_type="int", result_value="5")
        self.expect_expr("derived_with_dtor_but_no_base_dtor.foo()", result_type="int", result_value="6")

    def test_call_on_derived_as_base(self):
        self.common_setup()
        self.expect_expr("derived_with_dtor_as_base.foo()", result_type="int", result_value="3")
        self.expect_expr("derived_without_as_base.foo()", result_type="int", result_value="4")
        self.expect_expr("derived_with_base_dtor_as_base.foo()", result_type="int", result_value="5")
        self.expect_expr("derived_with_dtor_but_no_base_dtor_as_base.foo()", result_type="int", result_value="6")

    def test_call_overloaded(self):
        self.common_setup()
        self.expect("expr derived_with_overload.foo()", error=True, substrs=["too few arguments to function call, expected 1, have 0"])
        self.expect_expr("derived_with_overload.foo(1)", result_type="int", result_value="7")
        self.expect_expr("derived_with_overload_and_using.foo(1)", result_type="int", result_value="8")
        # FIXME: It seems the using declaration doesn't import the overload from the base class.
        self.expect("expr derived_with_overload_and_using.foo()", error=True, substrs=["too few arguments to function call, expected 1, have 0"])



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
import mlir.dialects.builtin as builtin
import mlir.dialects.std as std
import mlir.dialects.vector as vector

def run(f):
  print("\nTEST:", f.__name__)
  with Context(), Location.unknown():
    f()
  return f

# CHECK-LABEL: TEST: testPrintOp
@run
def testPrintOp():
  module = Module.create()
  with InsertionPoint(module.body):

    @builtin.FuncOp.from_py_func(VectorType.get((12, 5), F32Type.get()))
    def print_vector(arg):
      return vector.PrintOp(arg)

  # CHECK-LABEL: func @print_vector(
  # CHECK-SAME:                     %[[ARG:.*]]: vector<12x5xf32>) {
  #       CHECK:   vector.print %[[ARG]] : vector<12x5xf32>
  #       CHECK:   return
  #       CHECK: }
  print(module)


# CHECK-LABEL: TEST: testTransferReadOp
@run
def testTransferReadOp():
  module = Module.create()
  with InsertionPoint(module.body):
    vector_type = VectorType.get([2, 3], F32Type.get())
    memref_type = MemRefType.get([-1, -1], F32Type.get())
    index_type = IndexType.get()
    mask_type = VectorType.get(vector_type.shape, IntegerType.get_signless(1))
    identity_map = AffineMap.get_identity(vector_type.rank)
    identity_map_attr = AffineMapAttr.get(identity_map)
    func = builtin.FuncOp("transfer_read",
                          ([memref_type, index_type,
                            F32Type.get(), mask_type], []))
    with InsertionPoint(func.add_entry_block()):
      A, zero, padding, mask = func.arguments
      vector.TransferReadOp(vector_type, A, [zero, zero], identity_map_attr,
                            padding, mask, None)
      vector.TransferReadOp(vector_type, A, [zero, zero], identity_map_attr,
                            padding, None, None)
      std.ReturnOp([])

  # CHECK: @transfer_read(%[[MEM:.*]]: memref<?x?xf32>, %[[IDX:.*]]: index,
  # CHECK: %[[PAD:.*]]: f32, %[[MASK:.*]]: vector<2x3xi1>)
  # CHECK: vector.transfer_read %[[MEM]][%[[IDX]], %[[IDX]]], %[[PAD]], %[[MASK]]
  # CHECK: vector.transfer_read %[[MEM]][%[[IDX]], %[[IDX]]], %[[PAD]]
  # CHECK-NOT: %[[MASK]]
  print(module)
